# -*- coding: utf-8 -*-

# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from fixtures import MonkeyPatch

from config_tempest.clients import ClientManager
from config_tempest import main as tool
from config_tempest import tempest_conf
from config_tempest.tests.base import BaseConfigTempestTest


class TestOsClientConfigSupport(BaseConfigTempestTest):

    def setUp(self):
        super(TestOsClientConfigSupport, self).setUp()
        self.conf = self._get_conf("v2.0", "v3")

    def _check_credentials(self, manager, username, password, project_name):
        exp_user = manager.auth_provider.credentials._initial['username']
        exp_pass = manager.auth_provider.credentials._initial['password']
        creds = manager.auth_provider.credentials
        exp_project = creds._initial['project_name']
        self.assertEqual(exp_user, username)
        self.assertEqual(exp_pass, password)
        self.assertEqual(exp_project, project_name)

    def _override_setup(self):
        mock_function = mock.Mock(return_value={"id": "my_fake_id"})
        func2mock = ('config_tempest.clients.ProjectsClient.'
                     'get_project_by_name')
        self.useFixture(MonkeyPatch(func2mock, mock_function))

    def _obtain_client_config_data(self, non_admin=True, region_name=None):
        cloud_args = {
            'username': 'cloud_user',
            'password': 'cloud_pass',
            'project_name': 'cloud_project',
            'auth_url': 'http://auth.url.com/'
        }
        if region_name:
            cloud_args.update(region_name=region_name)
        # create an empty conf
        conf = tempest_conf.TempestConf()
        conf.set('identity', 'uri', cloud_args['auth_url'], priority=True)
        # call the function and check if data were obtained properly
        tool.set_cloud_config_values(non_admin, cloud_args, conf)
        if non_admin:
            self.assertEqual(cloud_args['username'],
                             conf.get('identity', 'username'))
            self.assertEqual(cloud_args['password'],
                             conf.get('identity', 'password'))
            self.assertEqual(cloud_args['project_name'],
                             conf.get('identity', 'project_name'))
        else:
            self.assertEqual(cloud_args['username'],
                             conf.get('auth', 'admin_username'))
            self.assertEqual(cloud_args['password'],
                             conf.get('auth', 'admin_password'))
            self.assertEqual(cloud_args['project_name'],
                             conf.get('auth', 'admin_project_name'))
        if region_name:
            self.assertEqual(cloud_args['region_name'],
                             conf.get('identity', 'region'))

    def test_init_manager_client_config(self):
        self._obtain_client_config_data()

    def test_init_manager_client_config_as_admin(self):
        self._obtain_client_config_data(non_admin=False)

    def test_init_manager_client_config_region_name(self):
        self._obtain_client_config_data(region_name='regionOne')

    def test_init_manager_client_config_get_default(self):
        manager = ClientManager(self.conf, self._get_creds(self.conf))
        # cloud_args is empty => check if default credentials were used
        self._check_credentials(manager,
                                self.conf.get('identity', 'username'),
                                self.conf.get('identity', 'password'),
                                self.conf.get('identity', 'project_name'))

    def test_init_manager_client_config_override(self):
        self._override_setup()
        manager = ClientManager(self.conf, self._get_creds(self.conf))
        # check if cloud_args credentials were overrided by the ones set in CLI
        self._check_credentials(manager,
                                self.conf.get('identity', 'username'),
                                self.conf.get('identity', 'password'),
                                self.conf.get('identity', 'project_name'))

    def test_init_manager_client_config_admin_override(self):
        self._override_setup()
        creds = self._get_creds(self.conf, admin=True)
        manager = ClientManager(self.conf, creds)
        # check if cloud_args credentials were overrided by admin ones
        self._check_credentials(manager,
                                self.conf.get('auth', 'admin_username'),
                                self.conf.get('auth', 'admin_password'),
                                self.conf.get('auth', 'admin_project_name'))



# Copyright 2013, 2016 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import re

from config_tempest import constants as C
from config_tempest.services.base import VersionedService

from tempest.lib import exceptions


class VolumeService(VersionedService):
    def set_extensions(self):
        body = self.do_get(self.service_url + '/extensions')
        body = json.loads(body)
        self.extensions = list(map(lambda x: x['alias'], body['extensions']))

    def set_versions(self):
        url, top_level = self.no_port_cut_url()
        body = self.do_get(url, top_level=top_level)
        self.versions_body = json.loads(body)
        self.versions = self.deserialize_versions(self.versions_body)

    def get_volume_pools(self):
        body = self.do_get(self.service_url + '/scheduler-stats/get_pools')
        body = json.loads(body)
        return body

    def set_default_tempest_options(self, conf):
        if 'v3' in self.service_url:
            m_vs = self.filter_api_microversions()
            conf.set('volume', 'min_microversion', m_vs['min_microversion'])
            conf.set('volume', 'max_microversion', m_vs['max_microversion'])

        try:
            pools = self.get_volume_pools()['pools']
        except exceptions.Forbidden:
            C.LOG.warning("User has no permissions to list back-end storage "
                          "pools - storage back-ends can't be discovered.")
            return
        if pools:
            backends = [
                re.findall(r'(\w*)@(\w*)', pool['name'])[0][1]
                for pool in pools
            ]
            conf.set('volume', 'backend_names', ','.join(backends))
            if len(backends) > 1:
                conf.set('volume-feature-enabled', 'multi_backend', 'True')

    def get_service_extension_key(self):
        return 'api_extensions'

    @staticmethod
    def get_codename():
        return 'cinder'

    def get_feature_name(self):
        return 'volume'

    def get_unversioned_service_type(self):
        return 'volume'

    @staticmethod
    def get_service_type():
        return ['volumev2', 'volumev3']

    def post_configuration(self, conf, is_service):
        # Verify if the cinder backup service is enabled
        if not is_service(name=self.name):
            C.LOG.info("No volume service found, "
                       "skipping backup service check")
            conf.set('volume-feature-enabled', 'backup', 'False')
            return
        try:
            params = {'binary': 'cinder-backup'}
            is_backup = self.client.list_services(**params)
        except exceptions.Forbidden:
            C.LOG.warning("User has no permissions to list services - "
                          "cinder-backup service can't be discovered.")
            conf.set('volume-feature-enabled', 'backup', 'False')
            return

        if is_backup:
            service = is_backup['services']
            if not service or service[0]['state'] == 'down':
                conf.set('volume-feature-enabled', 'backup', 'False')
            else:
                # post_configuration method is called with every volume (v2,
                # v3) service, therefore set the value with priority so that it
                # can't be overrided by this method called from other instance
                # of volume service
                conf.set('volume-feature-enabled', 'backup', 'True',
                         priority=True)



# Copyright 2016, 2017, 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import sys

from config_tempest import constants as C
from oslo_config import cfg
from six.moves import configparser
import tempest.config


class TempestConf(configparser.ConfigParser):
    # causes the config parser to preserve case of the options
    optionxform = str

    # set of pairs `(section, key)` which have a higher priority (are
    # user-defined) and will usually not be overwritten by `set()`
    priority_sectionkeys = set()

    CONF = tempest.config.TempestConfigPrivate(parse_conf=False)

    def __init__(self, write_credentials=True, **kwargs):
        self.write_credentials = write_credentials
        configparser.ConfigParser.__init__(self, **kwargs)

    def get_bool_value(self, value):
        """Returns boolean value of the string value given.

        :param value: True/False
        :type value: String
        :returns: Boolean value of the string value given
        :rtype: Boolean
        """
        strval = str(value).lower()
        if strval == 'true':
            return True
        elif strval == 'false':
            return False
        else:
            raise ValueError("'%s' is not a boolean" % value)

    def get_defaulted(self, section, key):
        """Returns default value for the section.key pair.

        :param section: a section in a tempest.conf file
        :type section: String
        :param key: a key in a section in a tempest.conf file
        :type key: String
        :returns: default value for the section.key pair
        :rtype: String
        """
        try:
            if self.has_option(section, key):
                return self.get(section, key)
            else:
                return self.CONF.get(section).get(key)
        except cfg.NoSuchOptError:
            C.LOG.warning("Option %s is not defined in %s section",
                          key, section)

    def set(self, section, key, value, priority=False):
        """Set value in configuration, similar to `ConfigParser.set`

        Creates non-existent sections. Keeps track of options which were
        specified by the user and should not be normally overwritten.

        :param section: a section in a tempest.conf file
        :type section: String
        :param key: a key in a section in a tempest.conf file
        :type key: String
        :param value: a value to be set to the section.key
        :type value: String
        :param priority: if True, always over-write the value. If False, don't
            over-write an existing value if it was written before with a
            priority (i.e. if it was specified by the user)
        :type priority: Boolean
        :returns: True if the value was written, False if not (because of
            priority)
        :rtype: Boolean
        """
        if not self.has_section(section) and section.lower() != "default":
            self.add_section(section)
        if not priority and (section, key) in self.priority_sectionkeys:
            C.LOG.debug("Option '[%s] %s = %s' was defined by user, NOT"
                        " overwriting into value '%s'", section, key,
                        self.get(section, key), value)
            return False
        if priority:
            self.priority_sectionkeys.add((section, key))
        C.LOG.debug("Setting [%s] %s = %s", section, key, value)
        configparser.ConfigParser.set(self, section, key, value)
        return True

    def write(self, out_path):
        C.LOG.info("Creating configuration file %s", os.path.abspath(out_path))
        if not self.write_credentials:
            C.LOG.info("Credentials will not be printed to a tempest.conf, "
                       "writing credentials is disabled.")
            self.remove_values(C.ALL_CREDENTIALS_KEYS)
        with open(out_path, 'w') as f:
            configparser.ConfigParser.write(self, f)

    def remove_values(self, to_remove):
        """Remove values from configuration file specified in arguments.

        :param to_remove: {'section.key': [values_to_be_removed], ...}
        :type to_remove: dict
        """
        for key_path in to_remove:
            section, key = key_path.split('.')
            try:
                conf_values = self.get(section, key).split(',')
                remove = to_remove[key_path]
                if len(remove) == 0:
                    # delete all values in section.key
                    self.remove_option(section, key)
                elif len(conf_values) == 1:
                    # make sure only the value specified by user
                    # will be deleted if in the key is other value
                    # than expected, ignore it
                    if conf_values[0] in to_remove[key_path]:
                        self.remove_option(section, key)
                else:
                    # exclude all unwanted values from the list
                    # and preserve the original order of items
                    conf_values = [v for v in conf_values if v not in remove]
                    self.set(section, key, ",".join(conf_values))
            except configparser.NoOptionError:
                # only inform a user, option specified by him doesn't exist
                C.LOG.error(sys.exc_info()[1])
            except configparser.NoSectionError:
                # only inform a user, section specified by him doesn't exist
                C.LOG.error(sys.exc_info()[1])

    def append_values(self, to_append):
        """Appends values to configuration file specified in arguments.

        :param to_append: {'section.key': [values_to_be_added], ...}
        :type to_append: dict
        """
        for key_path in to_append:
            section, key = key_path.split('.')
            try:
                conf_val = self.get(section, key).split(',')
                # omit duplicates if found any
                conf_val += list(set(to_append[key_path]) - set(conf_val))
                self.set(section, key, ",".join(conf_val))
            except configparser.NoOptionError:
                # only inform a user, option specified by him doesn't exist
                C.LOG.error(sys.exc_info()[1])
            except configparser.NoSectionError:
                # only inform a user, section specified by him doesn't exist
                C.LOG.error(sys.exc_info()[1])



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from config_tempest.services.services import Services
from config_tempest.tests.base import BaseConfigTempestTest
from unittest import mock


class TestServices(BaseConfigTempestTest):

    FAKE_ENTRY = {
        'endpoints': [
            {
                'region': 'my_region',
                'interface': 'public'
            },
            {
                'region': 'other_region',
                'interface': 'private'
            }
        ]
    }

    def setUp(self):
        super(TestServices, self).setUp()

    @mock.patch('config_tempest.services.services.Services.discover')
    @mock.patch('config_tempest.services.services.Services.'
                'set_catalog_and_url')
    @mock.patch('config_tempest.services.services.Services.'
                'get_available_services')
    def _create_services_instance(self, mock_avail, mock_catalog,
                                  mock_discover, v2=False):
        mock_avail.return_value = [{'name': 'my_service', 'type': 'my_type'}]
        conf = self._get_conf('v2', 'v3')
        creds = self._get_creds(conf, v2=v2)
        clients = mock.Mock()
        services = Services(clients, conf, creds)
        return services

    def test_get_endpoints_api_2(self):
        services = self._create_services_instance(v2=True)
        services._region = 'my_region'
        resp = services.get_endpoints(self.FAKE_ENTRY)
        self.assertEqual(resp, self.FAKE_ENTRY['endpoints'][0])
        services._region = 'doesnt_exist_region'
        resp = services.get_endpoints(self.FAKE_ENTRY)
        self.assertEqual(resp, self.FAKE_ENTRY['endpoints'][0])
        services._region = 'other_region'
        resp = services.get_endpoints(self.FAKE_ENTRY)
        self.assertEqual(resp, self.FAKE_ENTRY['endpoints'][1])

    def test_get_endpoints_api_3(self):
        services = self._create_services_instance()
        services._creds.api_version = 3
        services._region = 'my_region'
        resp = services.get_endpoints(self.FAKE_ENTRY)
        self.assertEqual(resp, self.FAKE_ENTRY['endpoints'][0])
        services._region = 'other_region'
        resp = services.get_endpoints(self.FAKE_ENTRY)
        self.assertEqual(resp, self.FAKE_ENTRY['endpoints'][0])

    def test_get_endpoints_no_endpoints(self):
        services = self._create_services_instance()
        resp = services.get_endpoints({'endpoints': []})
        self.assertEqual(resp, [])

    def test_parse_endpoints_empty(self):
        services = self._create_services_instance()
        services.public_url = "url"
        ep = []
        url = services.parse_endpoints(ep, "ServiceName")
        self.assertEqual("", url)

    def test_parse_endpoints(self):
        services = self._create_services_instance()
        services.public_url = "url"
        ep = {
            'url': 'http://10.0.0.107:8386/v1.1/96409a589d',
            'interface': 'public',
            'region': 'regioneOne',
        }
        url = services.parse_endpoints(ep, "ServiceName")
        self.assertEqual('http://10.0.0.107:8386/v1.1/96409a589d', url)
        ep['url'] = 'https://10.0.0.101/identity'
        auth_url = 'https://10.0.0.101:13000/v3.0'
        services._clients.auth_provider.auth_url = auth_url
        url = services.parse_endpoints(ep, 'ServiceName')
        self.assertEqual('https://10.0.0.101:13000/identity/v3', url)

    def test_parse_endpoints_not_ip_hostname(self):
        services = self._create_services_instance()
        services.public_url = "url"
        url = "https://identity-my.cloud.com:35456/v2.0"
        ep = {
            'url': url,
            'interface': 'public',
            'region': 'regioneOne',
        }
        services._clients.auth_provider.auth_url = "35456"
        url_resp = services.parse_endpoints(ep, "ServiceName")
        self.assertEqual(url, url_resp)

    def test_edit_identity_url_v3(self):
        services = self._create_services_instance()
        url_port = 'https://10.0.0.101:13000/v3.0'
        identity_url = 'https://10.0.0.101/identity'
        url_no_port = 'https://10.0.0.101/v333'
        services._clients.auth_provider.auth_url = url_port
        url = services.edit_identity_url(url_port)
        self.assertEqual(url_port, url)
        url = services.edit_identity_url(identity_url)
        self.assertEqual("https://10.0.0.101:13000/identity/v3", url)
        url = services.edit_identity_url(url_no_port)
        self.assertEqual(url_no_port, url)
        services._clients.auth_provider.auth_url = url_no_port
        url = services.edit_identity_url(identity_url)
        self.assertEqual(identity_url + "/v3", url)

    def test_edit_identity_url_v2(self):
        services = self._create_services_instance(v2=True)
        url_port = 'https://10.0.0.101:13000/v2.0'
        identity_url = 'https://10.0.0.101/identity'
        url_no_port = 'https://10.0.0.101/v2.0'
        services._clients.auth_provider.auth_url = url_port
        url = services.edit_identity_url(url_port)
        self.assertEqual(url_port, url)
        url = services.edit_identity_url(identity_url)
        self.assertEqual("https://10.0.0.101:13000/identity/v2", url)
        url = services.edit_identity_url(url_no_port)
        self.assertEqual(url_no_port, url)
        services._clients.auth_provider.auth_url = url_no_port
        url = services.edit_identity_url(identity_url)
        self.assertEqual(identity_url + "/v2", url)

    def test_get_service(self):
        services = self._create_services_instance()
        exp_resp = mock.Mock()
        exp_resp.s_type = 'my_service_type'
        services._services = [exp_resp]
        resp = services.get_service('my_service_type')
        self.assertEqual(resp, exp_resp)
        resp = services.get_service('my')
        self.assertEqual(resp, None)

    def test_is_service(self):
        services = self._create_services_instance()
        service = mock.Mock()
        service.name = 'my_service'
        service.s_type = 'my_type'
        services._services = [service]
        resp = services.is_service(name='my_service')
        self.assertEqual(resp, True)
        resp = services.is_service(name='other_service')
        self.assertEqual(resp, False)
        resp = services.is_service(**{'type': 'my_type'})
        self.assertEqual(resp, True)
        resp = services.is_service(**{'type': 'other_type'})
        self.assertEqual(resp, False)



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from config_tempest import constants as C
from config_tempest.services.base import Service
from tempest.lib import exceptions


class MeteringService(Service):

    def post_configuration(self, conf, is_service):
        try:
            params = {'type': 'metering'}
            services = self.client.list_services(**params)
        except exceptions.Forbidden:
            C.LOG.warning("User has no permissions to list services - "
                          "metering service can't be discovered.")
            return
        if services and len(services['services']):
            metering = services['services'][0]
            if 'ceilometer' in metering['name'] and metering['enabled']:
                conf.set('service_available', 'ceilometer', 'True')

    @staticmethod
    def get_service_type():
        return ['metering']

    @staticmethod
    def get_codename():
        return 'ceilometer'



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import yaml


def create_accounts_file(create, accounts_path, conf):
    if create:
        section = 'auth'
        prefix = 'admin_'
    else:
        section = 'identity'
        prefix = ''
    write_accounts_file(accounts_path,
                        conf.get(section, prefix + 'username'),
                        conf.get(section, prefix + 'password'),
                        conf.get(section, prefix + 'project_name'))


def write_accounts_file(path, username, password, project_name):
    """Creates a minimal accounts.yaml file.

    Dumps provided credentials in a yaml format.

    :type path: string
    :type username: string
    :type password: string
    :type project_name: string
    """
    comments = "# A minimal accounts.yaml file\n" \
               "# Will likely not work with swift, since additional\n" \
               "# roles are required. For more documentation see:\n" \
               "# https://opendev.org/openstack/tempest/src/branch/master/" \
               "etc/accounts.yaml.sample\n\n"
    accounts = []
    accounts.append({
        'username': username,
        'project_name': project_name,
        'password': password
    })
    with open(path, 'w') as outfile:
        for line in comments:
            outfile.write(line)
        yaml.safe_dump(accounts, outfile, default_flow_style=False)



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from config_tempest import profile
from config_tempest.tests.base import BaseConfigTempestTest


class TestProfile(BaseConfigTempestTest):

    def test_convert_remove_append(self):
        in_data = {
            'section.key1': 'value1',
            'section.key2': 'value1,value2'
        }
        expected = [
            'section.key1=value1',
            'section.key2=value1,value2'
        ]
        out_data = profile._convert_remove_append(in_data)
        self.assertItemsEqual(expected, out_data)

    @mock.patch('config_tempest.profile._read_yaml_file')
    def test_read_profile_file(self, mock_read_yaml):
        profile_data = {
            'create': True,
            'overrides': {
                'auth.use_dynamic_credentials': True
            },
            'append': {
                'network-feature-enabled.api_extensions': 'ext',
                'identity-feature-enabled.api_extensions': ['ext1', 'ext2'],
                'compute-feature-enabled.api_extensions': 'ext3,ext4'
            },
            'remove': {
                'network-feature-enabled.api_extensions': 'dvr',
                'identity-feature-enabled.api_extensions': ['dvr1', 'dvr2'],
                'compute-feature-enabled.api_extensions': 'dvr3,dvr4'
            },
            'network-id': 'network_id',
            'out': './etc/tempest.conf'
        }
        mock_read_yaml.return_value = profile_data
        ret_dict = profile.read_profile_file('path')
        expected = {
            'create': True,
            'remove': [
                'network-feature-enabled.api_extensions=dvr',
                'identity-feature-enabled.api_extensions=dvr1,dvr2',
                'compute-feature-enabled.api_extensions=dvr3,dvr4'
            ],
            'network-id': 'network_id',
            'overrides': [('auth', 'use_dynamic_credentials', 'True')],
            'append': [
                'network-feature-enabled.api_extensions=ext',
                'identity-feature-enabled.api_extensions=ext1,ext2',
                'compute-feature-enabled.api_extensions=ext3,ext4'
            ],
            'out': './etc/tempest.conf'
        }
        for key in ['create', 'network-id', 'out']:
            self.assertEqual(expected[key], ret_dict[key])
        for key in ['remove', 'overrides', 'append']:
            self.assertListEqual(sorted(expected[key]), sorted(ret_dict[key]))



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
from unittest import mock

from config_tempest import accounts
from config_tempest import main
from config_tempest.tests.base import BaseConfigTempestTest


class TestAccounts(BaseConfigTempestTest):
    """Accounts test class

    Tests for create_accounts_file and write_accounts_file methods.
    """

    def setUp(self):
        super(TestAccounts, self).setUp()
        self.conf = self._get_conf("v2.0", "v3")

    @mock.patch('config_tempest.accounts.write_accounts_file')
    def test_create_accounts_file(self, mock_write):
        path = "./etc/accounts.yaml"
        main.set_options(self.conf, None, False, "", accounts_path=path)
        # credentials under auth section
        accounts.create_accounts_file(True, path, self.conf)
        mock_write.assert_called_with(path, "admin", "adminPass",
                                      "adminProject")
        self.assertEqual(self.conf.get("auth", "test_accounts_file"),
                         os.path.abspath(path))

        # credentials under identity section
        accounts.create_accounts_file(False, path, self.conf)
        mock_write.assert_called_with(path, "demo", "secret", "demo")
        self.assertEqual(self.conf.get("auth", "test_accounts_file"),
                         os.path.abspath(path))



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from config_tempest import constants as C
from config_tempest.services.base import VersionedService

from tempest.lib import exceptions


class ShareService(VersionedService):

    def get_share_pools(self, detail=False):
        url = self.service_url + '/scheduler-stats/pools'
        if detail:
            url += '/detail'
        body = self.do_get(url)
        body = json.loads(body)
        return body

    def set_default_tempest_options(self, conf):
        if 'v2' in self.service_url:
            m_vs = self.filter_api_microversions()
            conf.set('share', 'min_api_microversion', m_vs['min_microversion'])
            conf.set('share', 'max_api_microversion', m_vs['max_microversion'])

        try:
            pools = self.get_share_pools(detail=True)['pools']
        except exceptions.Forbidden:
            C.LOG.warning("User has no permissions to list back-end storage "
                          "pools - storage back-ends can't be discovered.")
            return
        if pools:
            backends = []
            enable_protocols = []
            for pool in pools:
                backends.append(pool['backend'])
                protocol = pool['capabilities']['storage_protocol'].lower()
                enable_protocols.extend(protocol.split('_'))

            conf.set('share', 'backend_names', ','.join(backends))
            conf.set('share', 'enable_protocols', ','.join(enable_protocols))
            if len(backends) > 1:
                conf.set('share', 'multi_backend', 'True')

    def get_unversioned_service_type(self):
        return 'share'

    @staticmethod
    def get_codename():
        return 'manila'

    def get_feature_name(self):
        return 'share'

    @staticmethod
    def get_service_type():
        return ['share', 'sharev2']



# Copyright 2020 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from config_tempest.services.octavia import LoadBalancerService
from config_tempest.tests.base import BaseServiceTest


class TestOctaviaService(BaseServiceTest):
    def setUp(self):
        super(TestOctaviaService, self).setUp()
        self.conf = self._get_conf("v2", "v3")
        self.clients = self._get_clients(self.conf)
        self.Service = LoadBalancerService("ServiceName",
                                           "ServiceType",
                                           self.FAKE_URL + "v2.0/",
                                           self.FAKE_TOKEN,
                                           disable_ssl_validation=False)
        self.Service.client = self.FakeServiceClient(
            services={"services": [{"name": "octavia", "enabled": True}]}
        )
        self.conf.set("identity", "region", "regionOne")
        self._fake_service_do_get_method(self.FAKE_LBAAS_PROVIDERS)

    def test_list_drivers(self):
        expected_resp = [
            "amphora:The Octavia Amphora driver.",
            "octavia:Deprecated alias of the Octavia driver.",
        ]
        providers = self.Service.list_drivers()
        self.assertEqual(len(providers), len(expected_resp))

    @mock.patch("config_tempest.services.services.Services.is_service")
    def test_octavia_service_post_configuration(self, mock_is_service):
        mock_is_service.return_value = True
        self.Service.post_configuration(self.conf, mock_is_service)
        self.assertEqual(self.conf.get("load_balancer", "member_role"),
                         "member")
        self.assertEqual(self.conf.get("load_balancer", "region"),
                         "regionOne")
        self.assertEqual(self.conf.get("load_balancer",
                                       "enabled_provider_drivers"),
                         ("amphora:The Octavia Amphora driver.,"
                          "octavia:Deprecated alias of the Octavia driver."),
                         )



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from config_tempest.services.aws import Ec2Service
from config_tempest.services.aws import S3Service
from config_tempest.services.services import Services
from config_tempest.tests.base import BaseConfigTempestTest


class TestEc2Service(BaseConfigTempestTest):

    FAKE_URL = "http://10.200.16.10:8774/"

    @mock.patch('config_tempest.services.services.Services.discover')
    @mock.patch('config_tempest.services.services.Services.'
                'set_catalog_and_url')
    @mock.patch('config_tempest.services.services.Services.'
                'get_available_services')
    def setUp(self, mock_set_avail, mock_catalog, mock_discover):
        super(TestEc2Service, self).setUp()
        conf = self._get_conf('v2', 'v3')
        self.clients = self._get_clients(conf)
        self.Services = Services(self.clients, conf, self._get_creds(conf))

    def test_set_default_tempest_options(self):
        service = Ec2Service("ec2", "ec2", self.FAKE_URL, self.clients, False)
        service.set_default_tempest_options(self.Services._conf)
        ec2_url = self.Services._conf.get("aws", "ec2_url")
        self.assertEqual(ec2_url, self.FAKE_URL)


class TestS3Service(BaseConfigTempestTest):

    FAKE_URL = "http://10.200.16.10:8774/"

    @mock.patch('config_tempest.services.services.Services.discover')
    @mock.patch('config_tempest.services.services.Services.'
                'set_catalog_and_url')
    @mock.patch('config_tempest.services.services.Services.'
                'get_available_services')
    def setUp(self, mock_set_avail, mock_catalog, mock_discover):
        super(TestS3Service, self).setUp()
        conf = self._get_conf('v2', 'v3')
        self.clients = self._get_clients(conf)
        self.Services = Services(self.clients, conf, self._get_creds(conf))

    def test_set_default_tempest_options(self):
        service = S3Service("s3", "s3", self.FAKE_URL, self.clients, False)
        service.set_default_tempest_options(self.Services._conf)
        ec2_url = self.Services._conf.get("aws", "s3_url")
        self.assertEqual(ec2_url, self.FAKE_URL)



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from config_tempest.services.base import Service
from config_tempest.services.base import VersionedService
from config_tempest.tests.base import BaseServiceTest


class TestService(BaseServiceTest):
    def setUp(self):
        super(TestService, self).setUp()
        self.Service = Service("ServiceName",
                               "ServiceType",
                               self.FAKE_URL,
                               self.FAKE_TOKEN,
                               disable_ssl_validation=False)

    def _mocked_do_get(self, mock_urllib3):
        mock_http = mock_urllib3.PoolManager()
        expected_resp = mock_http.request('GET',
                                          self.FAKE_URL,
                                          self.FAKE_HEADERS)
        return expected_resp.data.decode('utf-8')

    @mock.patch('config_tempest.services.base.urllib3')
    def test_do_get(self, mock_urllib3):
        mock_http = mock.Mock()
        mock_r = mock.Mock()
        mock_r.status = 200
        mock_http.request.return_value = mock_r
        mock_urllib3.PoolManager.return_value = mock_http

        resp = self.Service.do_get(self.FAKE_URL)
        expected_resp = self._mocked_do_get(mock_urllib3)
        self.assertEqual(resp, expected_resp)

    def test_service_properties(self):
        self.assertEqual(self.Service.name, "ServiceName")
        self.assertEqual(self.Service.service_url, self.FAKE_URL)
        self.assertEqual(self.Service.headers, self.FAKE_HEADERS)
        self.assertEqual(self.Service.disable_ssl_validation, False)
        self.assertEqual(self.Service.client, None)
        self.assertEqual(self.Service.extensions, [])
        self.assertEqual(self.Service.versions, [])

    def test_set_extensions(self):
        self.Service.extensions = ['ext']
        self.Service.set_extensions()
        self.assertEqual(self.Service.extensions, [])

    def test_set_versions(self):
        self.Service.versions = ['ver']
        self.Service.set_versions()
        self.assertEqual(self.Service.versions, [])

    def test_get_extensions(self):
        self.Service.extensions = ['ext']
        self.assertEqual(self.Service.get_extensions(), ['ext'])

    def test_get_versions(self):
        self.Service.versions = ['ver']
        self.assertEqual(self.Service.get_versions(), ['ver'])


class TestVersionedService(BaseServiceTest):
    def setUp(self):
        super(TestVersionedService, self).setUp()
        self.Service = VersionedService("ServiceName",
                                        "ServiceType",
                                        self.FAKE_URL,
                                        self.FAKE_TOKEN,
                                        disable_ssl_validation=False)

    def test_set_get_versions(self):
        expected_resp = ['v2.0', 'v2.1']
        self._fake_service_do_get_method(self.FAKE_VERSIONS)
        self.Service.set_versions()
        resp = self.Service.get_versions()
        self.assertItemsEqual(resp, expected_resp)
        self.assertItemsEqual(self.Service.versions, expected_resp)

    def test_deserialize_versions(self):
        expected_resp = ['v2.0', 'v2.1']
        self._test_deserialize_versions(self.Service,
                                        expected_resp,
                                        self.FAKE_VERSIONS)

    def test_no_port_cut_url(self):
        resp = self.Service.no_port_cut_url()
        self.assertEqual(resp, (self.FAKE_URL, True))
        url = "http://10.200.16.10"
        self.Service.service_url = url
        resp = self.Service.no_port_cut_url()
        self.assertEqual(resp, (self.Service.service_url, False))
        self.Service.service_url = url + "/v3/cut/it/off"
        resp = self.Service.no_port_cut_url()
        self.assertEqual(resp, (url + '/', False))



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ssl import CertificateError
from unittest import mock

from fixtures import MonkeyPatch

from config_tempest.services import horizon
from config_tempest.tests.base import BaseConfigTempestTest


class TestConfigTempest(BaseConfigTempestTest):

    def setUp(self):
        super(TestConfigTempest, self).setUp()
        self.conf = self._get_conf("v2.0", "v3")

    def test_configure_horizon_ipv4(self):
        mock_function = mock.Mock(return_value=True)
        self.useFixture(MonkeyPatch('six.moves.urllib.request.urlopen',
                                    mock_function))
        horizon.configure_horizon(self.conf)
        self.assertEqual(self.conf.get('service_available', 'horizon'), "True")
        self.assertEqual(self.conf.get('dashboard', 'dashboard_url'),
                         "http://172.16.52.151/dashboard/")
        self.assertEqual(self.conf.get('dashboard', 'login_url'),
                         "http://172.16.52.151/dashboard/auth/login/")

    def test_configure_horizon_ipv6(self):
        mock_function = mock.Mock(return_value=True)
        self.useFixture(MonkeyPatch('six.moves.urllib.request.urlopen',
                                    mock_function))
        self.conf.set('identity', 'uri', 'http://[::1]:5000/v3', priority=True)
        horizon.configure_horizon(self.conf)
        self.assertEqual(self.conf.get('service_available', 'horizon'), "True")
        self.assertEqual(self.conf.get('dashboard', 'dashboard_url'),
                         "http://[::1]/dashboard/")
        self.assertEqual(self.conf.get('dashboard', 'login_url'),
                         "http://[::1]/dashboard/auth/login/")

    def test_configure_horizon_certificate_error(self):
        mock_function = mock.Mock(return_value=True)
        mock_function.side_effect = CertificateError
        self.useFixture(MonkeyPatch('six.moves.urllib.request.urlopen',
                                    mock_function))
        horizon.configure_horizon(self.conf)
        self.assertEqual(self.conf.get('service_available', 'horizon'),
                         "False")
        self.assertFalse(self.conf.has_section('dashboard'))



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from fixtures import MonkeyPatch

from config_tempest.services.image import ImageService
from config_tempest.tempest_conf import TempestConf
from config_tempest.tests.base import BaseServiceTest


class TestImageService(BaseServiceTest):

    CLIENT_MOCK = 'tempest.lib.services.image.v2.images_client.ImagesClient'
    IMAGES_LIST = [
        {"status": "active", "name": "ImageName"},
        {"status": "default", "name": "MyImage"},
        {"status": "active", "name": "MyImage"}
    ]

    def setUp(self):
        super(TestImageService, self).setUp()
        self.Service = ImageService("ServiceName",
                                    "ServiceType",
                                    self.FAKE_URL,
                                    self.FAKE_TOKEN,
                                    disable_ssl_validation=False)
        self.Service.disk_format = ".format"
        self.Service.non_admin = False
        self.Service.convert = False
        self.Service.client = self.FakeServiceClient()

        self.dir = "/img/"
        self.conf = TempestConf()
        self.conf.set("scenario", "img_dir", self.dir)
        self.conf.set("image", "image_path", "my_image.qcow2")
        self.conf.set("image", "http_image", "http_image.qcow2")

    @mock.patch('config_tempest.services.image.ImageService._find_image')
    @mock.patch('config_tempest.services.image.ImageService'
                '.find_or_upload_image')
    @mock.patch('os.makedirs')
    def _test_create_tempest_images(self, mock_makedirs, mock_find_upload,
                                    mock_find_image):
        mock_find_upload.side_effect = ["id_c", "id_d"]
        mock_find_image.return_value = {'name': 'my_image.qcow2'}
        self.Service.create_tempest_images(conf=self.conf)
        mock_makedirs.assert_called()
        self.assertEqual(self.conf.get('compute', 'image_ref'), 'id_c')
        self.assertEqual(self.conf.get('compute', 'image_ref_alt'), 'id_d')
        self.assertEqual(self.conf.get('scenario', 'img_file'),
                         'my_image.qcow2')

    @mock.patch('config_tempest.services.image.ImageService._find_image')
    @mock.patch('config_tempest.services.image.ImageService._download_file')
    @mock.patch('config_tempest.services.image.ImageService._upload_image')
    def _test_find_or_upload_image_not_found_creation_allowed_format(
            self, mock_upload_image,
            mock_download_file, mock_find_image, format):
        mock_find_image.return_value = None
        mock_upload_image.return_value = {"id": "my_fake_id"}
        image_source = format + "://any_random_url"
        image_dest = "my_dest"
        image_name = "my_image"
        image_id = self.Service.find_or_upload_image(
            image_id=None, image_dest=image_dest,
            image_name=image_name, image_source=image_source)
        mock_download_file.assert_called_with(image_source, image_dest)
        mock_upload_image.assert_called_with(image_name, image_dest)
        self.assertEqual(image_id, "my_fake_id")

    def _mock_list_images(self, return_value, image_name, expected_resp):

        mock_function = mock.Mock(return_value=return_value)
        func2mock = self.FAKE_CLIENT_MOCK + '.list_images'
        self.useFixture(MonkeyPatch(func2mock, mock_function))
        resp = self.Service._find_image(image_id=None,
                                        image_name=image_name)
        self.assertEqual(resp, expected_resp)

    def test_set_get_versions(self):
        exp_resp = ['v2.0', 'v2.1']
        self._set_get_versions(self.Service, exp_resp, self.FAKE_VERSIONS)

    @mock.patch('config_tempest.services.image.ImageService'
                '.find_or_upload_image')
    def test_create_tempest_images_exception(self, mock_find_upload):
        mock_find_upload.side_effect = Exception
        exc = Exception
        self.assertRaises(exc,
                          self.Service.create_tempest_images,
                          conf=self.conf)

    def test_create_tempest_images_ref_alt_ref(self):
        self.conf.set('compute', 'image_ref', 'id_a')
        self.conf.set('compute', 'image_ref_alt', 'id_b')
        self._test_create_tempest_images()

    def test_create_tempest_images_ref_no_alt_ref(self):
        self.conf.set('compute', 'image_ref', 'id_a')
        self._test_create_tempest_images()

    def test_create_tempest_images_no_ref_alt_ref(self):
        self.conf.set('compute', 'image_ref_alt', 'id_b')
        self._test_create_tempest_images()

    def test_create_tempest_images_no_ref_no_alt_ref(self):
        self._test_create_tempest_images()

    @mock.patch('config_tempest.services.image.ImageService._find_image')
    def test_find_or_upload_image_not_found_creation_not_allowed(
            self, mock_find_image):
        mock_find_image.return_value = None
        exc = Exception
        self.assertRaises(exc, self.Service.find_or_upload_image,
                          image_id=None, image_name=None)

    def test_find_or_upload_image_not_found_creation_allowed_http(self):
        self._test_find_or_upload_image_not_found_creation_allowed_format(
            format="http")

    def test_find_or_upload_image_not_found_creation_allowed_https(self):
        self._test_find_or_upload_image_not_found_creation_allowed_format(
            format="https")

    @mock.patch('shutil.copyfile')
    @mock.patch('config_tempest.services.image.ImageService._find_image')
    @mock.patch('config_tempest.services.image.ImageService._download_file')
    @mock.patch('config_tempest.services.image.ImageService._upload_image')
    def test_find_or_upload_image_not_found_creation_allowed_ftp_old(
            self, mock_upload_image, mock_download_file, mock_find_image,
            mock_copy):
        mock_find_image.return_value = None
        mock_upload_image.return_value = {"id": "my_fake_id"}
        # image source does not start with http or https
        image_source = "ftp://any_random_url"
        image_dest = "place_on_disk"
        image_name = "my_image"
        image_id = self.Service.find_or_upload_image(
            image_id=None, image_name=image_name,
            image_source=image_source, image_dest=image_dest)
        mock_copy.assert_called_with(image_source, image_dest)
        mock_upload_image.assert_called_with(
            image_name, image_dest)
        self.assertEqual(image_id, "my_fake_id")

    @mock.patch('os.path.isfile')
    @mock.patch('config_tempest.services.image.ImageService._find_image')
    def test_find_or_upload_image_found_downloaded(
            self, mock_find_image, mock_isfile):
        mock_find_image.return_value = \
            {"status": "active", "name": "ImageName", "id": "my_fake_id"}
        mock_isfile.return_value = True
        image_id = self.Service.find_or_upload_image(
            image_id=None, image_name=None)
        self.assertEqual(image_id, "my_fake_id")

    @mock.patch('config_tempest.services.image.ImageService._download_image')
    @mock.patch('os.path.isfile')
    @mock.patch('config_tempest.services.image.ImageService._find_image')
    def test_find_or_upload_image_found_not_downloaded(
            self, mock_find_image, mock_isfile, mock_download_image):
        image_id = "my_fake_id"
        mock_find_image.return_value = \
            {"status": "active", "name": "ImageName", "id": image_id}
        mock_isfile.return_value = False
        image_id = self.Service.find_or_upload_image(
            image_id=None, image_name=None)
        mock_download_image.assert_called()
        self.assertEqual(image_id, "my_fake_id")

    def test_find_image_found(self):
        expected_resp = {"status": "default", "name": "MyImage"}
        self._mock_list_images(return_value={"images": self.IMAGES_LIST},
                               image_name="MyImage",
                               expected_resp=expected_resp)

    def test_find_image_not_found(self):
        self._mock_list_images(return_value={"images": self.IMAGES_LIST},
                               image_name="DoesNotExist",
                               expected_resp=None)

    def test_find_image_by_id(self):
        expected_resp = {"id": "001", "status": "active", "name": "ImageName"}
        mock_function = mock.Mock(return_value=expected_resp)
        func2mock = self.FAKE_CLIENT_MOCK + '.show_image'
        self.useFixture(MonkeyPatch(func2mock, mock_function))
        resp = self.Service._find_image(image_id="001",
                                        image_name="cirros")
        self.assertEqual(resp, expected_resp)

    @mock.patch('subprocess.call')
    @mock.patch('os.path.exists')
    def test_convert_image_to_raw(self, mock_exists, mock_subcall):
        path = '/path/of/my/image.qcow2'
        raw_path = '/path/of/my/image.raw'
        mock_exists.return_value = False
        mock_subcall.return_value = 0
        self.Service.convert_image_to_raw(path)
        mock_subcall.assert_called_with(['qemu-img', 'convert',
                                        path, raw_path])
        self.assertEqual(self.Service.disk_format, 'raw')



# Copyright 2013, 2016, 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import os

LOG = logging.getLogger(__name__)

# Get the current tempest workspace path
TEMPEST_WORKSPACE = os.getcwd()

DEPLOYER_INPUT = os.path.join(os.path.expanduser("~"),
                              "tempest-deployer-input.conf")
DEFAULT_IMAGE = ("https://download.cirros-cloud.net/0.4.0/"
                 "cirros-0.4.0-x86_64-disk.img")
DEFAULT_IMAGE_FORMAT = 'qcow2'

DEFAULT_FLAVOR_RAM = 128
DEFAULT_FLAVOR_RAM_ALT = 192
DEFAULT_FLAVOR_DISK = 1
DEFAULT_FLAVOR_VCPUS = 1

# The dict holds the credentials, which are not supposed to be printed
# to a tempest.conf when --test-accounts CLI parameter is used.
ALL_CREDENTIALS_KEYS = {
    "auth.admin_username": [],
    "auth.admin_password": [],
    "auth.admin_project_name": [],
    "auth.admin_domain_name": [],
    "identity.username": [],
    "identity.password": [],
    "identity.project_name": [],
    "identity.alt_username": [],
    "identity.alt_password": [],
    "identity.alt_project_name": [],
    "identity.admin_domain_name": [],
}



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from config_tempest.services.base import Service


class Ec2Service(Service):

    def set_default_tempest_options(self, conf):
        conf.set('aws', 'ec2_url', self.service_url)

    @staticmethod
    def get_service_type():
        return ['ec2']


class S3Service(Service):

    def set_default_tempest_options(self, conf):
        conf.set('aws', 's3_url', self.service_url)

    @staticmethod
    def get_service_type():
        return ['s3']



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re
from six.moves import urllib


def get_base_url(endpoint):
    """Return the base url.

    :param endpoint: endpoint
    :type endpoint: string
    :returns: base_url - without version
    :rtype: string
    """
    url = urllib.parse.urlsplit(endpoint)
    # the regex matches a version in the url path
    url_path_without_version = re.split(r'(^|/)+v\d+(\.\d+)?', url.path)[0]
    url = list(url)
    url[2] = url_path_without_version + '/'
    return urllib.parse.urlunsplit(url)



# -*- coding: utf-8 -*-

# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from fixtures import MonkeyPatch
import json
import logging
from oslotest import base

from config_tempest.clients import ClientManager
from config_tempest.credentials import Credentials
from config_tempest import tempest_conf

# disable logging when running unit tests
logging.disable(logging.CRITICAL)


class BaseConfigTempestTest(base.BaseTestCase):
    """Test case base class for all config_tempest unit tests"""

    FAKE_V3_VERSIONS = (
        [{
            'status': 'stable',
            'id': 'v3.8',
        }, {
            'status': 'deprecated',
            'id': 'v2.0',
        }]
    )
    FAKE_V2_VERSIONS = (
        [{
            'status': 'deprecated',
            'id': 'v3.8',
        }, {
            'status': 'stable',
            'id': 'v2.0',
        }]
    )

    def _get_conf(self, V2, V3):
        """Creates fake conf for testing purposes"""
        conf = tempest_conf.TempestConf()
        uri = "http://172.16.52.151:5000/"
        conf.set("identity", "username", "demo")
        conf.set("identity", "password", "secret")
        conf.set("identity", "project_name", "demo")
        conf.set("identity", "disable_ssl_certificate_validation", "true")
        conf.set("identity", "auth_version", "v3")
        conf.set("identity", "uri", uri + V2, priority=True)
        conf.set("identity", "uri_v3", uri + V3)
        conf.set("auth", "admin_username", "admin")
        conf.set("auth", "admin_project_name", "adminProject")
        conf.set("auth", "admin_password", "adminPass")
        conf.set("auth", "use_dynamic_credentials", "False", priority=True)
        return conf

    def _get_alt_conf(self, V2, V3):
        """Contains newer params in place of the deprecated params"""
        conf = tempest_conf.TempestConf()
        uri = "http://172.16.52.151:5000/"
        conf.set("identity", "username", "demo")
        conf.set("identity", "password", "secret")
        conf.set("identity", "project_name", "demo")
        conf.set("identity", "disable_ssl_certificate_validation", "true")
        conf.set("identity", "auth_version", "v3")
        conf.set("identity", "uri", uri + V2, priority=True)
        conf.set("identity", "uri_v3", uri + V3)
        conf.set("auth", "admin_username", "admin")
        conf.set("auth", "admin_project_name", "adminProject")
        conf.set("auth", "admin_password", "adminPass")
        conf.set("auth", "use_dynamic_credentials", "True", priority=True)
        return conf

    def _get_creds(self, conf, admin=False, v2=False):
        # We return creds configured to v2 or v3
        func2mock = 'config_tempest.credentials.Credentials._list_versions'
        return_value = self.FAKE_V3_VERSIONS
        if v2:
            return_value = self.FAKE_V2_VERSIONS
        mock_function = mock.Mock(return_value=return_value)
        self.useFixture(MonkeyPatch(func2mock, mock_function))
        return Credentials(conf, admin)

    def _get_clients(self, conf, creds=None):
        """Returns ClientManager instance"""
        if creds is None:
            creds = self._get_creds(conf, v2=True)
        return ClientManager(conf, creds)


class BaseServiceTest(BaseConfigTempestTest):
    """Test case base class for all api_discovery unit tests"""

    FAKE_TOKEN = "s6d5f45sdf4s564f4s6464sdfsd514"
    FAKE_CLIENT_MOCK = 'config_tempest.tests.base.BaseServiceTest' + \
                       '.FakeServiceClient'
    FAKE_HEADERS = {
        'Accept': 'application/json', 'X-Auth-Token': FAKE_TOKEN
    }
    FAKE_URL = "http://10.200.16.10:8774/"
    FAKE_VERSIONS = (
        {
            "versions": [{
                "status": "SUPPORTED",
                "updated": "2011-01-21T11:33:21Z",
                "links": [{
                    "href": "http://10.200.16.10:8774/v2 / ",
                    "rel": "self "
                }],
                "min_version": "",
                "version": "",
                "id": "v2.0",
                "values": [
                    {"id": 'v3.8'}
                ]
            }, {
                "status": "CURRENT",
                "updated": "2013-07-23T11:33:21Z",
                "links": [{
                    "href": "http://10.200.16.10:8774/v2.1/",
                    "rel": "self"
                }],
                "min_version": "2.1",
                "version": "2.41",
                "id": "v2.1",
                "values": [
                    {"id": 'v2.0'}
                ]
            }, {
                "status": "DEPRECATED",
                "updated": "2013-07-23T11:33:21Z",
                "links": [{
                    "href": "http://10.200.16.10:8774/v1/",
                    "rel": "self"
                }],
                "min_version": "1.0",
                "version": "1.0",
                "id": "v1.0",
                "values": [
                    {"id": 'v1.0'}
                ]
            }]
        }
    )
    FAKE_IDENTITY_VERSIONS = (
        {
            'versions': {
                'values': [{
                    'status': 'stable',
                    'id': 'v3.8',
                }, {
                    'status': 'deprecated',
                    'id': 'v2.0',
                }]
            }
        }
    )
    FAKE_IDENTITY_VERSION = (
        {
            'version': {
                'status': 'stable',
                'id': 'v3.8',
            }
        }
    )
    FAKE_EXTENSIONS = (
        {
            "extensions": [{
                "updated": "2014-12-03T00:00:00Z",
                "name": "Multinic",
                "namespace": "http://docs.openstack.org/compute/ext/fake_xml",
                "alias": "NMN",
                "description": "Multiple network support."
            }, {
                "updated": "2014-12-03T00:00:00Z",
                "name": "DiskConfig",
                "namespace": "http://docs.openstack.org/compute/ext/fake_xml",
                "alias": "OS-DCF",
                "description": "Disk Management Extension."
            }]
        }
    )
    FAKE_IDENTITY_EXTENSIONS = (
        {
            "extensions": {
                'values': [{
                    'alias': 'OS-DCF',
                    'id': 'v3.8',
                }, {
                    'alias': 'NMN',
                    'id': 'v2.0',
                }]
            }
        }
    )
    FAKE_LBAAS_PROVIDERS = (
        {
            "providers": [{
                "name": "amphora",
                "description": "The Octavia Amphora driver."
            }, {
                "name": "octavia",
                "description": "Deprecated alias of the Octavia driver."
            }]
        }
    )
    FAKE_STORAGE_EXTENSIONS = (
        {
            "formpost": {},
            "methods": ["GET", "HEAD", "PUT", "POST", "DELETE"],
            "ratelimit": {
                "account_ratelimit": 0.0,
                "max_sleep_time_seconds": 60.0,
                "container_ratelimits": []
            },
            "account_quotas": {},
            "swift": {
                "container_listing_limit": 10000,
                "allow_account_management": True,
                "max_container_name_length": 256
            }
        }
    )
    FAKE_ROLES = (
        {
            "roles": [
                {
                    "name": "ResellerAdmin",
                    "id": "2b4df7a671a443d741c62b4df7a671a443d741c6",
                    "domain_id": None
                }
            ]
        }
    )
    FAKE_ACCOUNTS = (
        {
            'status': '200',
            u'content-length': '2',
            'content-location': 'http://192.168.122.120:8080/healthcheck',
            u'connection': 'close',
            u'x-trans-id': 'txec03483c96cd4958a5c6b-005b17c346',
            u'date': 'Wed, 06 Jun 2018 11:19:34 GMT',
            u'content-type': 'text/plain',
            u'x-openstack-request-id': 'txec03483c96cd4958a5c6b-005b17c346'
        },
        'OK')

    class FakeRequestResponse(object):
        URL = 'https://docs.openstack.org/api/openstack-identity/3/ext/'
        FAKE_V3_EXTENSIONS = (
            {
                'resources': {
                    URL + 'OS-INHERIT/1.0/rel/domain_user_' +
                    'role_inherited_to_projects': "",

                    URL + 'OS-SIMPLE-CERT/1.0/rel/ca_certificate': "",

                    URL + 'OS-EP-FILTER/1.0/rel/endpoint_group_to_' +
                    'project_association': "",

                    URL + 'OS-EP-FILTER/1.0/rel/project_endpoint': "",

                    URL + 'OS-OAUTH1/1.0/rel/user_access_token_roles': ""
                }
            }
        )

        def __init__(self, bytes_content=False):
            self.content = json.dumps(self.FAKE_V3_EXTENSIONS)
            if bytes_content:
                self.content = self.content.encode('utf-8')

    class FakeServiceClient(object):
        def __init__(self, services=None):
            self.client = mock.Mock()
            self.return_value = mock.Mock()
            self.services = services

        def list_networks(self):
            return self.return_value

        def list_services(self, **kwargs):
            return self.services

    def _fake_service_do_get_method(self, fake_data):
        function2mock = 'config_tempest.services.base.Service.do_get'
        do_get_output = json.dumps(fake_data)
        mocked_do_get = mock.Mock()
        mocked_do_get.return_value = do_get_output
        self.useFixture(MonkeyPatch(function2mock, mocked_do_get))

    def _set_get_extensions(self, service, expected_resp, fake_data):
        # mock do_get response
        self._fake_service_do_get_method(fake_data)
        # set the fake extensions
        service.set_extensions()
        # check if extensions were set
        self.assertItemsEqual(service.extensions, expected_resp)
        # check if get method returns the right data
        resp = service.get_extensions()
        self.assertItemsEqual(resp, expected_resp)

    def _set_get_versions(self, service, expected_resp, fake_data):
        # mock do_get response
        self._fake_service_do_get_method(fake_data)
        # set the fake versions
        service.set_versions()
        # check if versions were set
        self.assertItemsEqual(service.versions, expected_resp)
        # check if get method returns the right data
        resp = service.get_versions()
        self.assertItemsEqual(resp, expected_resp)

    def _test_deserialize_versions(self, service, expected_resp, fake_data):
        resp = service.deserialize_versions(fake_data)
        self.assertItemsEqual(resp, expected_resp)

    def _assert_conf_get_not_raises(self, exc, section, value):
        try:
            self.conf.get(section, value)
        except exc:
            return
        self.assertTrue(False)



# -*- coding: utf-8 -*-

# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from config_tempest import tempest_conf
from config_tempest.tests.base import BaseConfigTempestTest


class TestTempestConf(BaseConfigTempestTest):
    def setUp(self):
        super(TestTempestConf, self).setUp()
        self.conf = tempest_conf.TempestConf()

    def test_set_value(self):
        conf = self._get_conf("v2.0", "v3")
        resp = conf.set("section", "key", "value")
        self.assertTrue(resp)
        self.assertEqual(conf.get("section", "key"), "value")
        self.assertEqual(conf.get_defaulted("section", "key"), "value")

    def test_set_value_overwrite(self):
        conf = self._get_conf("v2.0", "v3")
        # set value without priority (default: priority=False)
        resp = conf.set("section", "key", "value")
        self.assertTrue(resp)
        self.assertEqual(conf.get("section", "key"), "value")
        # value should be overwritten, because it wasn't set with priority
        resp = conf.set("section", "key", "new_value")
        self.assertTrue(resp)
        self.assertEqual(conf.get("section", "key"), "new_value")

    def test_set_value_overwrite_priority(self):
        conf = self._get_conf("v2.0", "v3")
        resp = conf.set("sectionPriority", "key", "value", priority=True)
        resp = conf.set("sectionPriority", "key", "new_value")
        self.assertFalse(resp)
        self.assertEqual(conf.get("sectionPriority", "key"), "value")

    def test_set_value_overwrite_by_priority(self):
        conf = self._get_conf("v2.0", "v3")
        resp = conf.set("section", "key", "value")
        self.assertTrue(resp)
        self.assertEqual(conf.get("section", "key"), "value")
        resp = conf.set("section", "key", "new_value", priority=True)
        self.assertTrue(resp)
        self.assertEqual(conf.get("section", "key"), "new_value")

    def test_set_value_overwrite_priority_by_priority(self):
        conf = self._get_conf("v2.0", "v3")
        resp = conf.set("sectionPriority", "key", "value", priority=True)
        self.assertTrue(resp)
        self.assertEqual(conf.get("sectionPriority", "key"), "value")
        resp = conf.set("sectionPriority", "key", "new_value", priority=True)
        self.assertTrue(resp)
        self.assertEqual(conf.get("sectionPriority", "key"), "new_value")

    def test_get_bool_value(self):
        self.assertTrue(self.conf.get_bool_value("True"))
        self.assertFalse(self.conf.get_bool_value("False"))
        self.assertRaises(ValueError, self.conf.get_bool_value, "no")

    def test_remove_values(self):
        api_exts = "router_availability_zone,rbac-policies,pagination,sorting,"
        api_exts += "standard-attr-description,router,binding,metering,"
        api_exts += "allowed-address-pairs,project-id,dvr,l3-flavors,tag-ext"
        remove_exts = ["router", "project-id", "dvr"]
        remove = {
            "identity.username": ["demo"],
            "identity.project_name": ["project"],
            "compute.image_ssh_user": ["rhel", "cirros"],
            "network-feature-enabled.api_extensions": remove_exts
        }
        conf = self._get_conf("v2.0", "v3")
        conf.set("compute", "image_ssh_user", "cirros")
        conf.set("network-feature-enabled", "api_extensions", api_exts)
        conf.remove_values(remove)
        self.assertFalse(conf.has_option("identity", "username"))
        self.assertTrue(conf.has_option("identity", "project_name"))
        self.assertFalse(conf.has_option("compute", "image_ssh_user"))
        conf_exts = conf.get("network-feature-enabled", "api_extensions")
        conf_exts = conf_exts.split(',')
        for ext in api_exts.split(','):
            if ext in remove_exts:
                self.assertNotIn(ext, conf_exts)
            else:
                self.assertIn(ext, conf_exts)

    def test_remove_values_having_hyphen(self):
        api_exts = "dvr,l3-flavors,rbac-policies,project-id"
        remove_exts = ["dvr", "project-id"]
        remove = {
            "network-feature-enabled.api_extensions": remove_exts
        }
        conf = self._get_conf("v2.0", "v3")
        conf.set("network-feature-enabled", "api_extensions", api_exts)
        conf.remove_values(remove)
        conf_exts = conf.get("network-feature-enabled", "api_extensions")
        conf_exts = conf_exts.split(',')
        for ext in api_exts.split(','):
            if ext in remove_exts:
                self.assertNotIn(ext, conf_exts)
            else:
                self.assertIn(ext, conf_exts)

    @mock.patch('config_tempest.tempest_conf.C.LOG')
    def test_remove_not_defined_values(self, mock_logging):
        self.conf.remove_values({"notExistSection.key": []})
        # check if LOG.error was called
        self.assertTrue(mock_logging.error.called)
        self.conf.remove_values({"section.notExistKey": []})
        # check if LOG.error was called
        self.assertTrue(mock_logging.error.called)

    def test_append_values(self):
        api_exts = "dvr,l3-flavors,rbac-policies"
        add_exts = ["dvr", "project-id"]
        add = {
            "compute-feature-enabled.api_extensions": add_exts
        }
        self.conf = self._get_conf("v2.0", "v3")
        self.conf.set("compute-feature-enabled", "api_extensions", api_exts)
        self.conf.append_values(add)
        conf_exts = self.conf.get("compute-feature-enabled", "api_extensions")
        conf_exts = conf_exts.split(',')
        self.assertEqual(len(conf_exts), 4)
        self.assertIn("project-id", conf_exts)

    def test_append_values_with_overrides(self):
        # Test if --add option can override an option which was
        # passed to python-tempestconf as an override, it shouldn't
        api_exts = "dvr,l3-flavors,rbac-policies"
        add_exts = ["dvr", "project-id"]
        add = {
            "compute-feature-enabled.api_extensions": add_exts
        }
        self.conf = self._get_conf("v2.0", "v3")
        # let's simulate a situation when the following apis were set
        # via overrides => they are set with the priority
        self.conf.set("compute-feature-enabled", "api_extensions",
                      api_exts, priority=True)
        self.conf.append_values(add)
        conf_exts = self.conf.get("compute-feature-enabled", "api_extensions")
        conf_exts = conf_exts.split(',')
        # if there are still 3 extensions, no new was added
        self.assertEqual(len(conf_exts), 3)
        # option added via --add shouldn't be there
        self.assertNotIn("project-id", conf_exts)
        self.assertIn("dvr", conf_exts)
        self.assertIn("l3-flavors", conf_exts)
        self.assertIn("rbac-policies", conf_exts)



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import yaml


def _convert_remove_append(options):
    """Convert append and remove dicts read from .yaml file.

    :param options: {'section.key': 'value1,value2', ...}
    :type options: dict
    :return: ['section.key=value1,value2', ...]
    :rtype: list
    """
    converted = []
    for key in options:
        v = options[key]
        if isinstance(v, list):
            v = ','.join(v)
        converted.append(key + "=" + v)
    return converted


def _read_yaml_file(path):
    """Read a .yaml file.

    :param path: path to the profile.yaml file
    :type path: string
    :return: profile arguments
    :rtype: dict
    """
    with open(path, 'r') as stream:
        return yaml.safe_load(stream)


def read_profile_file(path):
    """Read python-tempestconf arguments from a .yaml file.

    :param path: path to the profile.yaml file
    :type path: string
    :return: profile arguments
    :rtype: dict
    """
    profile_args = _read_yaml_file(path)
    # convert overrides, to a list of tuples (s, k, v)
    overrides = []
    if 'overrides' in profile_args:
        for key in profile_args['overrides']:
            s, k = key.split('.')
            v = profile_args['overrides'][key]
            if isinstance(v, list):
                v = ','.join(v)
            overrides.append((s, k, str(v)))
    profile_args['overrides'] = overrides
    # convert remove
    remove = []
    if 'remove' in profile_args:
        remove = _convert_remove_append(profile_args['remove'])
    profile_args['remove'] = remove
    # convert append values
    append = []
    if 'append' in profile_args:
        append = _convert_remove_append(profile_args['append'])
    profile_args['append'] = append
    return profile_args


def generate_profile(args, path):
    """Generates a sample profile.yaml file.

    :type args: argparse.Namespace
    :param path: Specifies where the sample file will be generated in.
    :type path: string
    """
    iterable_args = vars(args)
    # pop following arguments as they are written
    # more detailed to the file below
    iterable_args.pop('append')
    iterable_args.pop('overrides')
    iterable_args.pop('remove')
    # pop profile as that shouldn't be in a profile.yaml
    iterable_args.pop('profile')
    with open(path, 'w') as outfile:
        yaml.safe_dump(iterable_args, outfile, default_flow_style=False)
        outfile.write("""append: {}
  #identity.username: username
  #compute-feature-enabled.api_extensions:
  #  - dvr
  #  - extension
overrides: {}
  #identity.username: username
  #identity.password:
  #  - my_password
  #compute-feature-enabled.api_extensions:
  #  - dvr
  #  - extension
remove: {}
  #identity.username: username
  #compute-feature-enabled.api_extensions:
  #  - dvr
  #  - extension
""")



# Copyright 2016, 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from six.moves import configparser
from tempest.lib import exceptions

from config_tempest.constants import LOG
from config_tempest.services.base import Service


class ObjectStorageService(Service):
    def set_extensions(self):
        if 'v3' not in self.service_url:  # it's not a v3 url
            try:
                body = self.do_get(self.service_url, top_level=True,
                                   top_level_path="info")
                body = json.loads(body)
                # Remove Swift general information from extensions list
                body.pop('swift')
                self.extensions = body.keys()
            except Exception:
                self.extensions = []
        else:
            self.extensions = []

    def list_create_roles(self, conf, client):
        try:
            roles = client.list_roles()['roles']
            for section_key in ["operator_role", "reseller_admin_role"]:
                key_value = conf.get_defaulted("object-storage", section_key)
                if key_value not in [r['name'] for r in roles]:
                    LOG.info("Creating %s role", key_value)
                    try:
                        client.create_role(name=key_value)
                    except exceptions.Conflict:
                        LOG.info("Role %s already exists", key_value)
            conf.set('object-storage', 'operator_role', 'admin')
        except exceptions.Forbidden:
            LOG.info("Roles can't be listed or created. The user doesn't have "
                     "permissions.")
            # If is not admin, we set the operator_role to Member
            # otherwise we set to admin
            conf.set('object-storage', 'operator_role', 'Member')

    def get_feature_name(self):
        return 'object-storage'

    def get_service_extension_key(self):
        return 'discoverable_apis'

    def _check_health_check(self, path):
        try:
            self.client.accounts.skip_path()
            resp, _ = self.client.accounts.get(path, {})
            return resp['status'] == '200'
        except Exception as e:
            LOG.warning('Healthcheck API not discovered giving %s', e)
            return False

    def check_service_status(self, conf):
        """Use healthcheck api to check the service status

        :type conf: TempestConf object
        """
        # Check for swift discoverability if it is False
        # check_service_status returns False
        # Else above is True, then we can check for healthcheck
        # API then we can find the service_status
        try:
            if not conf.get_bool_value(
                conf.get(
                    'object-storage-feature-enabled',
                    'discoverability')):
                return False
            return True
        except configparser.NoSectionError:
            # On swift, healthcheck is under http://.../healthcheck, while in
            # ceph is under http://.../swift/healthcheck, we check both cases
            return_value = self._check_health_check('healthcheck')
            if not return_value:
                return_value = self._check_health_check('swift/healthcheck')
            return return_value

    def set_default_tempest_options(self, conf):
        """Set default values for swift

        :type conf: TempestConf object
        """
        swift_status = self.check_service_status(conf)
        # Set roles based on service status
        if swift_status:
            self.list_create_roles(conf, self.client.roles)

    @staticmethod
    def get_service_type():
        return ['object-store']

    @staticmethod
    def get_codename():
        return 'swift'



# Copyright 2013 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import requests

from six.moves import urllib

from config_tempest.constants import LOG
from config_tempest.services.base import VersionedService


class IdentityService(VersionedService):
    def __init__(self, name, s_type, service_url, token,
                 disable_ssl_validation, client=None, **kwargs):
        super(IdentityService, self).__init__(
            name, s_type, service_url, token, disable_ssl_validation,
            client, **kwargs)
        self.extensions_v3 = []
        version = ''
        if 'v2' in self.service_url:
            version = '/v2.0'
            url_parse = urllib.parse.urlparse(self.service_url)
            self.service_url = '{}://{}{}'.format(url_parse.scheme,
                                                  url_parse.netloc, version)

    def set_extensions(self):
        if 'v2' in self.service_url:
            body = self.do_get(self.service_url + '/extensions')
            body = json.loads(body)
            values = body['extensions']['values']
            self.extensions = list(map(lambda x: x['alias'], values))
            return
        # Keystone api changed in v3, the concept of extensions changed. Right
        # now, all the existing extensions are part of keystone core api, so,
        # there's no longer the /extensions endpoint. The extensions that are
        # stable, are enabled by default, the ones marked as experimental are
        # disabled by default. Checking the tempest source, there's no test
        # pointing to extensions endpoint, so I am very confident that this
        # will not be an issue. If so, we need to list all the /OS-XYZ
        # extensions to identify what is enabled or not. This would be a manual
        # check every time keystone change, add or delete an extension, so I
        # rather prefer to set empty list here for now.
        self.extensions = []

    def get_service_extension_key(self):
        return 'api_extensions'

    def get_supported_versions(self):
        return ['v2', 'v3']

    @staticmethod
    def get_service_type():
        return ['identity']

    def set_identity_v3_extensions(self):
        """Returns discovered identity v3 extensions

        As keystone V3 uses a JSON Home to store the extensions.
        This method implements a different discovery method.

        :return: A list with the discovered extensions
        """
        try:
            r = requests.get(self.service_url,
                             verify=False,
                             headers={'Accept': 'application/json-home'})
            # check for http status
            r.raise_for_status()
        except requests.exceptions.HTTPError:
            LOG.warning("Request on service '%s' with url '%s' failed, "
                        "checking for v3", 'identity', self.service_url)
            if 'v3' not in self.service_url:
                self.service_url = self.service_url + '/v3'
                r = requests.get(self.service_url,
                                 verify=False,
                                 headers={'Accept': 'application/json-home'})

        ext_h = 'https://docs.openstack.org/api/openstack-identity/3/ext/'
        content = r.content.decode('utf-8')
        res = [x for x in json.loads(content)['resources'].keys()]
        ext = [ex for ex in res if 'ext' in ex]
        ext = [str(e).replace(ext_h, '').split('/')[0] for e in ext]
        self.extensions_v3 = list(set(ext))

    def set_versions(self):
        super(IdentityService, self).set_versions(top_level=False)

    def get_extensions(self):
        all_ext_lst = self.extensions + self.extensions_v3
        return list(set(all_ext_lst))

    def deserialize_versions(self, body):
        try:
            versions = []
            for v in body['versions']['values']:
                # TripleO is in transition to v3 only, so the environment
                # still returns v2 versions even though they're deprecated.
                # Therefor pick only versions with stable status.
                if v['status'] == 'stable':
                    versions.append(v['id'])
            return versions
        except KeyError:
            return [body['version']['id']]

    def set_default_tempest_options(self, conf):
        """Set keystone feature flags based upon version ID."""
        supported_versions = self.get_versions()

        if supported_versions:
            major, minor = supported_versions[0].split('.')
            conf.set('identity', 'auth_version', major)

        if len(supported_versions) <= 1:
            return
        for version in supported_versions:
            major, minor = version.split('.')[:2]
            # Enable the domain specific roles feature flag.
            # For more information see:
            # https://docs.openstack.org/api-ref/identity/v3
            if major == 'v3' and int(minor) >= 6:
                conf.set('identity-feature-enabled',
                         'forbid_global_implied_dsr',
                         'True')



# Copyright 2019 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from six.moves import configparser

from config_tempest.constants import LOG
from config_tempest.services.base import Service
from config_tempest.users import Users


class OrchestrationService(Service):

    @staticmethod
    def get_service_type():
        return ['orchestration']

    @staticmethod
    def get_codename():
        return 'heat'

    def set_default_tempest_options(self, conf):
        try:
            sec = 'heat_plugin'

            conf.set(sec, 'username', conf.get('identity', 'username'))
            conf.set(sec, 'password', conf.get('identity', 'password'))
            conf.set(sec, 'admin_username', conf.get('auth', 'admin_username'))
            conf.set(sec, 'admin_password', conf.get('auth', 'admin_password'))
            conf.set(sec, 'project_name', conf.get('identity', 'project_name'))
            admin_project_name = conf.get('auth', 'admin_project_name')
            conf.set(sec, 'admin_project_name', admin_project_name)
            conf.set(sec, 'region', conf.get('identity', 'region'))

            v = '3' if conf.get('identity', 'auth_version') == 'v3' else '2'
            if v == '3':
                conf.set(sec, 'auth_url', conf.get('identity', 'uri_v3'))
            else:
                conf.set(sec, 'auth_url', conf.get('identity', 'uri'))
            conf.set(sec, 'auth_version', v)

            domain_name = conf.get('auth', 'admin_domain_name')
            conf.set(sec, 'project_domain_name', domain_name)
            conf.set(sec, 'user_domain_name', domain_name)
        except configparser.NoOptionError:
            LOG.warning("Be aware that an option required for "
                        "heat_tempest_plugin cannot be set!")

        networks_client = self.client['networks']
        subnets_client = self.client['subnets']
        projects_client = self.client['projects']
        roles_client = self.client['roles']
        users_client = self.client['users']

        heat_network_name = "heat_tempestconf_network"
        heat_subnet_name = "heat_tempestconf_subnet"
        project = conf.get('identity', 'project_name')

        try:
            network_list = networks_client.list_networks()
            heat_network = [network for network in network_list['networks']
                            if network['name'] == heat_network_name]

            if not heat_network:
                project_id = projects_client.get_project_by_name(project)['id']
                heat_network = networks_client.create_network(
                    name=heat_network_name,
                    project_id=project_id)
                heat_network_id = heat_network['network']['id']
                subnets_client.create_subnet(
                    network_id=heat_network_id,
                    ip_version=4,
                    cidr="192.168.199.0/24",
                    name=heat_subnet_name)

            conf.set(sec, 'fixed_network_name', heat_network_name)
        except Exception:
            LOG.warning("Could not create network within the %s project "
                        "needed by heat tempest plugin!", project)

        try:
            users = Users(projects_client, roles_client, users_client, conf)
            username = conf.get('identity', 'username')
            users.give_role_to_user(username, "member")
        except Exception:
            LOG.warning("Could not assign role 'member' to user '%s'!",
                        username)

    def post_configuration(self, conf, is_service):
        conf.set('heat_plugin', 'disable_ssl_certificate_validation',
                 str(self.disable_ssl_validation))
        if self.ca_certs:
            conf.set('heat_plugin', 'ca_file', self.ca_certs)
        if conf.has_section('compute'):
            compute_options = conf.options('compute')
            if 'flavor_ref' in compute_options:
                conf.set('heat_plugin', 'minimal_instance_type',
                         conf.get('compute', 'flavor_ref'))
            if 'flavor_ref_alt' in compute_options:
                conf.set('heat_plugin', 'instance_type',
                         conf.get('compute', 'flavor_ref_alt'))
            if 'image_ref' in compute_options:
                conf.set('heat_plugin', 'minimal_image_ref',
                         conf.get('compute', 'image_ref'))
            if 'image_ref_alt' in compute_options:
                conf.set('heat_plugin', 'image_ref',
                         conf.get('compute', 'image_ref_alt'))
        if conf.has_section('network'):
            network = conf.get('network', 'floating_network_name')
            conf.set('heat_plugin', 'network_for_ssh', network)
            conf.set('heat_plugin', 'floating_network_name', network)



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from config_tempest.services import volume
from config_tempest.tempest_conf import TempestConf
from config_tempest.tests.base import BaseServiceTest


class TestVolumeService(BaseServiceTest):
    def setUp(self):
        super(TestVolumeService, self).setUp()
        self.Service = volume.VolumeService("ServiceName",
                                            "ServiceType",
                                            self.FAKE_URL,
                                            self.FAKE_TOKEN,
                                            disable_ssl_validation=False)
        self.conf = TempestConf()

    def test_set_get_extensions(self):
        exp_resp = ['NMN', 'OS-DCF']
        self._set_get_extensions(self.Service, exp_resp, self.FAKE_EXTENSIONS)

    def test_set_get_versions(self):
        exp_resp = ['v2.0', 'v2.1']
        self._set_get_versions(self.Service, exp_resp, self.FAKE_VERSIONS)

    @mock.patch('config_tempest.services.services.Services.is_service')
    @mock.patch('config_tempest.services.volume.C.LOG')
    def test_post_configuration_no_volume(self, mock_logging, mock_is_service):
        mock_is_service.return_value = False
        self.Service.post_configuration(self.conf, mock_is_service)
        self.assertTrue(mock_logging.info.called)
        self.assertEqual(self.conf.get('volume-feature-enabled', 'backup'),
                         'False')

    @mock.patch('config_tempest.services.services.Services.is_service')
    def test_post_configuration_state_down(self, mock_is_service):
        client_service_mock = self.FakeServiceClient(services={
            'services': [
                {
                    "state": "down"
                }
            ]
        })
        self.Service.client = client_service_mock
        mock_is_service.return_value = True
        self.Service.post_configuration(self.conf, mock_is_service)
        self.assertEqual(self.conf.get('volume-feature-enabled',
                         'backup'), 'False')

    @mock.patch('config_tempest.services.services.Services.is_service')
    def test_post_configuration_no_service(self, mock_is_service):
        client_service_mock = self.FakeServiceClient(services={
            'services': []
        })
        self.Service.client = client_service_mock
        mock_is_service.return_value = True
        self.Service.post_configuration(self.conf, mock_is_service)
        self.assertEqual(self.conf.get('volume-feature-enabled',
                         'backup'), 'False')



# Copyright 2016, 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
This script will generate the etc/tempest.conf file by applying a series of
specified options in the following order:

1. Default values provided by the tool.

2. Values using the file provided by the --deployer-input argument to the
script.
Some required options differ among deployed clouds but the right values cannot
be discovered by the user. The file used here could be created by an installer,
or manually if necessary.

3. Values provided in client's cloud config file or as an environment
variables, see documentation of openstacksdk
https://docs.openstack.org/openstacksdk/latest/

4. Values provided on the command line. These override all other values.

5. Discovery. Values that have not been provided in steps [2-4] will be
obtained by querying the cloud.
"""

import argparse
import logging
import os
import sys

import openstack
from oslo_config import cfg
from six.moves import configparser

from config_tempest import accounts
from config_tempest.clients import ClientManager
from config_tempest import constants as C
from config_tempest.constants import LOG
from config_tempest.credentials import Credentials
from config_tempest.flavors import Flavors
from config_tempest import profile
from config_tempest.services.services import Services
from config_tempest.tempest_conf import TempestConf
from config_tempest.users import Users


def set_logging(debug, verbose):
    """Set logging based on the arguments.

    :type debug: Boolean
    :type verbose: Boolean
    """
    log_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    logging.basicConfig(format=log_format)
    if debug:
        LOG.setLevel(logging.DEBUG)
    elif verbose:
        LOG.setLevel(logging.INFO)


def load_basic_defaults(conf):
    """Load basic default options into conf file.

    :type conf: TempestConf object
    """
    LOG.debug("Setting basic default values")
    default_values = {
        "DEFAULT": [
            ("debug", "true"),
            ("use_stderr", "false"),
            ("log_file", "tempest.log")
        ],
        "identity": [
            ("username", "demo_tempestconf"),
            ("password", "secrete"),
            ("project_name", "demo"),
            ("alt_username", "alt_demo_tempestconf"),
            ("alt_password", "secrete"),
            ("alt_project_name", "alt_demo")
        ],
        "scenario": [
            ("img_dir", "etc")
        ],
        "auth": [
            ("tempest_roles", "_member_"),
            ("admin_username", "admin"),
            ("admin_project_name", "admin"),
            ("admin_domain_name", "Default")
        ],
        "object-storage": [
            ("reseller_admin_role", "ResellerAdmin")
        ],
        "oslo-concurrency": [
            ("lock_path", "/tmp")
        ],
        "compute-feature-enabled": [
            # Default deployment does not use shared storage
            ("live_migration", "false"),
            ("live_migrate_paused_instances", "true"),
            ("preserve_ports", "true")
        ],
        "network-feature-enabled": [
            ("ipv6_subnet_attributes", "true")
        ]}

    for section in default_values.keys():
        if section != "DEFAULT" and not conf.has_section(section):
            conf.add_section(section)
        for key, value in default_values[section]:
            if not conf.has_option(section, key):
                conf.set(section, key, value)


def read_deployer_input(deployer_input_file, conf):
    """Read deployer-input file and set values in conf accordingly.

    :param deployer_input_file: Path to the deployer inut file
    :type deployer_input_file: String
    :param conf: TempestConf object
    """
    LOG.info("Adding options from deployer-input file '%s'",
             deployer_input_file)
    deployer_input = configparser.ConfigParser()
    deployer_input.read(deployer_input_file)
    for section in deployer_input.sections():
        # There are no deployer input options in DEFAULT
        for (key, value) in deployer_input.items(section):
            conf.set(section, key, value, priority=True)


def set_options(conf, deployer_input, non_admin, image_path, overrides=[],
                accounts_path=None, cloud_creds=None,
                no_default_deployer=False):
    """Set options in conf provided by different source.

    1. read the default values
    2. read a file provided by --deployer-input argument
    3. read default DEPLOYER_INPUT if --no-deployer-input is False and no
       deployer_input was passed
    4. set values from client's config (openstacksdk support) if provided
    5. set overrides - may override values which were set in the steps above

    :param conf: TempestConf object
    :param deployer_input: Path to the deployer inut file
    :type deployer_input: string
    :type non_admin: boolean
    :param image_path: An image to be uploaded to glance
    :type image_path: string
    :param overrides: list of tuples: [(section, key, value)]
    :type overrides: list
    :param accounts_path: A path where accounts.yaml is or will be created.
    :type accounts_path: string
    :param cloud_creds: Cloud credentials from client's config
    :type cloud_creds: dict
    """
    load_basic_defaults(conf)
    # image.image_path is a python-tempestconf option which defines which
    # image will be uploaded to glance
    conf.set('image', 'image_path', image_path)

    if deployer_input and os.path.isfile(deployer_input):
        LOG.info("Reading deployer input from file %s", deployer_input)
        read_deployer_input(deployer_input, conf)
    elif os.path.isfile(C.DEPLOYER_INPUT) and not no_default_deployer:
        LOG.info("Reading deployer input from file %s", C.DEPLOYER_INPUT)
        read_deployer_input(C.DEPLOYER_INPUT, conf)

    if non_admin:
        # non admin, so delete auth admin values which were set
        # in load_basic_defaults method
        conf.set("auth", "admin_username", "")
        conf.set("auth", "admin_project_name", "")
        conf.set("auth", "admin_password", "")
        conf.set("auth", "use_dynamic_credentials", "False", priority=True)

    # get and set auth data from client's config
    if cloud_creds:
        set_cloud_config_values(non_admin, cloud_creds, conf)

    if accounts_path:
        # new way for running using accounts file
        conf.set("auth", "use_dynamic_credentials", "False", priority=True)
        conf.set("auth", "test_accounts_file",
                 os.path.abspath(accounts_path))

    # set overrides - values specified in CLI
    for section, key, value in overrides:
        conf.set(section, key, value, priority=True)

    uri = conf.get("identity", "uri")
    if "v3" in uri:
        conf.set("identity", "uri_v3", uri)
    else:
        # TODO(arxcruz) make a check if v3 is enabled
        conf.set("identity", "uri_v3", uri.replace("v2.0", "v3"))


def get_arg_parser():
    parser = argparse.ArgumentParser(__doc__)
    cloud_config = openstack.config.OpenStackConfig()
    cloud_config.register_argparse_arguments(parser, sys.argv)
    parser.add_argument('--create', action='store_true', default=False,
                        help="""Create Tempest resources
                                Make *python-tempestconf* to create Tempest
                                resources such as flavors needed for running
                                Tempest tests.""")
    parser.add_argument('--out', default="etc/tempest.conf",
                        help="""Output file
                                A name of the file where the discovered Tempest
                                configuration will be written to.""")
    parser.add_argument('--deployer-input', default=None,
                        help="""Path to deployer file
                                A file in the format of tempest.conf that will
                                override the default values. It is usually
                                created by an installer and contains
                                environment specific options.

                                The deployer-input file is an alternative to
                                providing key/value pairs. If there are also
                                key/value pairs they will be applied after the
                                deployer-input file.

                                If the option is **not defined** and
                                **--no-default-deployer** is **not used**,
                                python-tempestconf **will try** to look for the
                                file in `$HOME/tempest-deployer-input.conf`
                                location.""")
    parser.add_argument('--no-default-deployer', action='store_true',
                        default=False,
                        help="""Do not check for the default deployer input in
                                `$HOME/tempest-deployer-input.conf`""")
    parser.add_argument('overrides', nargs='*', default=[],
                        help="""Override options
                                Key value pairs used to hardcode values in
                                `tempest.conf`. The key is a section.key where
                                section is a section header in the conf file.
                                For example:
                                 $ discover-tempest-config \\
                                  identity.username myname \\
                                  identity.password mypass""")
    parser.add_argument('--debug', action='store_true', default=False,
                        help='Print debugging information.')
    parser.add_argument('--verbose', '-v', action='store_true', default=False,
                        help='Print more information about the execution.')
    parser.add_argument('--no-rng', action='store_true', default=False,
                        help="""Create new flavors and upload images without
                                random number generator device.""")
    parser.add_argument('--non-admin', action='store_true', default=False,
                        help="""Simulate non-admin credentials.
                                When True, the credentials are used as
                                non-admin ones. No resources are created.""")
    parser.add_argument('--test-accounts', default=None, metavar='PATH',
                        help="""Tempest accounts.yaml file
                                Defines a path to a Tempest accounts.yaml
                                file.
                                For example:
                                 --test-accounts $HOME/tempest/accounts.yaml
                             """)
    parser.add_argument('--create-accounts-file', default=None,
                        metavar='PATH',
                        help="""Generate Tempest accounts file
                                Minimal accounts file will be created in the
                                specified path.
                                For example:
                                  --create-accounts-file $HOME/accounts.yaml
                             """)
    parser.add_argument('--profile', default=None, metavar='PATH',
                        help="""python-tempestconf's profile.yaml file
                                A file which contains definition of
                                python-tempestconf's arguments.
                                NOTE: If this argument is used, other
                                arguments cannot be defined!""")
    parser.add_argument('--generate-profile', default=None,
                        metavar='PATH',
                        help="""Generate a sample profile.yaml file.
                                A sample profile.yaml will be generated in the
                                specified path. After that python-tempestconf
                                ends.
                                For example:
                                  --generate-profile $HOME/profile.yaml
                             """)
    parser.add_argument('--image-disk-format', default=C.DEFAULT_IMAGE_FORMAT,
                        help="""A format of an image to be uploaded to glance.
                                Default is '%s'""" % C.DEFAULT_IMAGE_FORMAT)
    parser.add_argument('--image', default=C.DEFAULT_IMAGE,
                        help="""An image name/path/url to be uploaded to
                                glance if it's not already there. The name of
                                the image is the leaf name of the path. Default
                                is '%s'""" % C.DEFAULT_IMAGE)
    parser.add_argument('--flavor-min-mem', default=C.DEFAULT_FLAVOR_RAM,
                        type=int, help="""Specify minimum memory for new
                        flavours, default is '%s'.""" % C.DEFAULT_FLAVOR_RAM)
    parser.add_argument('--flavor-min-disk', default=C.DEFAULT_FLAVOR_DISK,
                        type=int, help="""Specify minimum disk size for new
                        flavours, default is '%s'.""" % C.DEFAULT_FLAVOR_DISK)
    parser.add_argument('--convert-to-raw', action='store_true', default=False,
                        help="""Convert images to raw format before uploading
                                to glance.""")
    parser.add_argument('--network-id',
                        help="""Specify which network with external connectivity
                                should be used by the tests.""")
    parser.add_argument('--append', action='append', default=[],
                        metavar="SECTION.KEY=VALUE[,VALUE]",
                        help="""Append values to tempest.conf
                                Key value pair to be appended to the
                                configuration file.
                                NOTE: Multiple values are supposed to be
                                divided by a COLON only, WITHOUT spaces.
                                For example:
                                 $ discover-tempest-config \\
                                  --append features.ext=tag[,tag-ext] \\
                                  --append section.ext=ext[,another-ext]
                             """)
    parser.add_argument('--remove', action='append', default=[],
                        metavar="SECTION.KEY=VALUE[,VALUE]",
                        help="""Remove values from tempest.conf
                                Key value pair to be removed from the
                                configuration file.
                                NOTE: Multiple values are supposed to be
                                divided by a COLON only, WITHOUT spaces.
                                For example:
                                 $ discover-tempest-config \\
                                  --remove identity.username=myname \\
                                  --remove feature-enabled.api_ext=http[,https]
                             """)
    return parser


def parse_arguments():
    parser = get_arg_parser()
    args = parser.parse_args()
    if args.create and args.non_admin:
        raise Exception("Options '--create' and '--non-admin' cannot be used"
                        " together, since creating" " resources requires"
                        " admin rights")
    if args.test_accounts and args.create_accounts_file:
        raise Exception("Options '--test-accounts' and "
                        "'--create-accounts-file' can't be used together.")
    args.overrides = parse_overrides(args.overrides)
    return args


def parse_values_to_remove(options):
    """Manual parsing of remove arguments.

    :param options: list of arguments following --remove argument
    :return: dictionary containing key paths with values to be removed
    :rtype: dict
    EXAMPLE: {'identity.username': [myname],
              'identity-feature-enabled.api_extensions': [http, https]}
    """
    parsed = {}
    for argument in options:
        if len(argument.split('=')) == 2:
            section, values = argument.split('=')
            if len(section.split('.')) != 2:
                raise Exception("Missing dot. The option --remove has to "
                                "come in the format 'section.key=value[,value"
                                "]', but got '%s'." % argument)
            parsed[section] = values.split(',')
        else:
            # missing equal sign, all values in section.key will be deleted
            parsed[argument] = []
    return parsed


def parse_values_to_append(options):
    """Manual parsing of --append arguments.

    :param options: list of arguments following --append argument.
    :return: dictionary containing key paths with values to be added
    :rtype: dict
    """
    parsed = {}
    for argument in options:
        if len(argument.split('=')) == 2:
            section, values = argument.split('=')
            if len(section.split('.')) != 2:
                raise Exception("Missing dot. The option --append has to "
                                "come in the format 'section.key=value[,value"
                                "]', but got '%s'." % argument)
            if values == '':
                raise Exception("No values to append specified. The option "
                                "--append has to come in the format "
                                "'section.key=value[, value]', but got "
                                "'%s'" % values)
            parsed[section] = values.split(',')
        else:
            # missing equal sign, no values to add were specified, if a user
            # wants to just create a section, it can be done so via overrides
            raise Exception("Missing equal sign or more than just one found.")
    return parsed


def parse_overrides(overrides):
    """Manual parsing of positional arguments.

    :param overrides: list of section.keys and values to override, example:
               ['section.key', 'value', 'section.key', 'value']
    :return: list of tuples, example: [('section', 'key', 'value'), ...]
    :rtype: list
    """
    if len(overrides) % 2 != 0:
        raise Exception("An odd number of override options was found. The"
                        " overrides have to be in 'section.key value' format.")
    i = 0
    new_overrides = []
    while i < len(overrides):
        section_key = overrides[i].split('.')
        value = overrides[i + 1]
        if len(section_key) != 2:
            raise Exception("Missing dot. The option overrides has to come in"
                            " the format 'section.key value', but got '%s'."
                            % (overrides[i] + ' ' + value))
        section, key = section_key
        new_overrides.append((section, key, value))
        i += 2
    return new_overrides


def set_cloud_config_values(non_admin, cloud_creds, conf):
    """Set values from client's cloud config file.

    Set admin and non-admin credentials and uri from cloud credentials.
    Note: the values may be later overridden by values specified in CLI.

    :type non_admin: Boolean
    :param cloud_creds: auth data from openstacksdk
    :type cloud_creds: dict
    :param conf: TempestConf object
    """
    try:
        if non_admin:
            # Tempest doesn't have non-admin credentials, but we're gonna
            # keep them under identity for future usage
            conf.set('identity', 'username', cloud_creds['username'])
            conf.set('identity',
                     'project_name',
                     cloud_creds['project_name'])
            conf.set('identity', 'password', cloud_creds['password'])
        else:
            # admin credentials are under auth section
            conf.set('auth', 'admin_username', cloud_creds['username'])
            conf.set('auth',
                     'admin_project_name',
                     cloud_creds['project_name'])
            conf.set('auth', 'admin_password', cloud_creds['password'])
        conf.set('identity', 'uri', cloud_creds['auth_url'])

        if 'region_name' in cloud_creds:
            conf.set('identity', 'region', cloud_creds['region_name'])
    except cfg.NoSuchOptError:
        LOG.warning(
            'Could not load some identity options from cloud config file')


def get_cloud_creds(args_namespace):
    """Get cloud credentials based on argument namespace.

    If args contains --os-cloud argument, the method returns cloud
    credentials related to that cloud, otherwise, returns credentials
    of the current cloud.

    :type args_namespace: argparse.Namespace
    :return: cloud credentials
    :rtype: dict
    EXAMPLE: {'username': 'demo', 'project_name': 'demo',
              'user_domain_name': 'Default',
              'auth_url': 'http://172.16.52.8:5000/v3',
              'password': 'f0921edc3c2b4fc8', 'project_domain_name': 'Default'}
    """
    if args_namespace.os_cloud:
        cloud = openstack.connect(cloud=args_namespace.os_cloud)
    else:
        cloud = openstack.connect(argparse=args_namespace)

    cloud_creds = cloud.config.get_auth_args()
    region_name = cloud.config.config['region_name']
    if region_name:
        cloud_creds['region_name'] = region_name

    request_args = cloud.config.get_requests_verify_args()
    cloud_creds['request_args'] = {
        'verify': request_args[0],
        'cert': request_args[1],
    }

    return cloud_creds


def config_tempest(**kwargs):
    # convert a list of remove values to a dict
    remove = parse_values_to_remove(kwargs.get('remove', []))
    add = parse_values_to_append(kwargs.get('append', []))
    set_logging(kwargs.get('debug', False), kwargs.get('verbose', False))

    accounts_path = kwargs.get('test_accounts')
    if kwargs.get('create_accounts_file') is not None:
        accounts_path = kwargs.get('create_accounts_file')
    conf = TempestConf(write_credentials=accounts_path is None)
    set_options(conf, kwargs.get('deployer_input'),
                kwargs.get('non_admin', False),
                kwargs.get('image_path', C.DEFAULT_IMAGE),
                kwargs.get('overrides', []),
                accounts_path,
                kwargs.get('cloud_creds'))

    request_args = kwargs.get('cloud_creds', {}).get('request_args', {})
    credentials = Credentials(conf, not kwargs.get('non_admin', False),
                              **request_args)
    clients = ClientManager(conf, credentials)

    if kwargs.get('create', False) and kwargs.get('test_accounts') is None:
        users = Users(clients.projects, clients.roles, clients.users, conf)
        users.create_tempest_users()

    services = Services(clients, conf, credentials)

    if services.is_service(**{"type": "compute"}):
        flavors = Flavors(clients.flavors, kwargs.get('create', False), conf,
                          kwargs.get('flavor_min_mem', C.DEFAULT_FLAVOR_RAM),
                          kwargs.get('flavor_min_disk', C.DEFAULT_FLAVOR_DISK),
                          no_rng=kwargs.get('no_rng', False))
        flavors.create_tempest_flavors()

    if services.is_service(**{"type": "image"}):
        image = services.get_service('image')
        image.set_image_preferences(kwargs.get('image_disk_format',
                                               C.DEFAULT_IMAGE_FORMAT),
                                    kwargs.get('non_admin', False),
                                    no_rng=kwargs.get('no_rng', False),
                                    convert=kwargs.get('convert_to_raw',
                                                       False))
        image.create_tempest_images(conf)

    if services.is_service(**{"type": "network"}):
        network = services.get_service("network")
        network.create_tempest_networks(conf, kwargs.get('network_id'))

    services.post_configuration()
    services.set_supported_api_versions()
    services.set_service_extensions()

    if accounts_path is not None and kwargs.get('test_accounts') is None:
        LOG.info("Creating an accounts.yaml file in: %s", accounts_path)
        accounts.create_accounts_file(kwargs.get('create', False),
                                      accounts_path,
                                      conf)

    # remove all unwanted values if were specified
    if remove != {}:
        LOG.info("Removing configuration: %s", str(remove))
        conf.remove_values(remove)
    if add != {}:
        LOG.info("Adding configuration: %s", str(add))
        conf.append_values(add)
    out_path = kwargs.get('out', 'etc/tempest.conf')
    conf.write(out_path)


def main():
    args = parse_arguments()
    if args.generate_profile:
        profile.generate_profile(args, args.generate_profile)
        sys.exit(0)
    if args.profile:
        profile_args = profile.read_profile_file(args.profile)
        # update default args by values gained from the profile
        # Namespace can't be updated, so translate it to a dict first
        args_dict = vars(args)
        args_dict.update(profile_args)
        args = argparse.Namespace(**args_dict)
    cloud_creds = get_cloud_creds(args)
    config_tempest(
        append=args.append,
        cloud_creds=cloud_creds,
        convert_to_raw=args.convert_to_raw,
        create=args.create,
        create_accounts_file=args.create_accounts_file,
        debug=args.debug,
        deployer_input=args.deployer_input,
        flavor_min_mem=args.flavor_min_mem,
        flavor_min_disk=args.flavor_min_disk,
        image_disk_format=args.image_disk_format,
        image_path=args.image,
        network_id=args.network_id,
        non_admin=args.non_admin,
        no_rng=args.no_rng,
        os_cloud=args.os_cloud,
        out=args.out,
        overrides=args.overrides,
        remove=args.remove,
        test_accounts=args.test_accounts,
        verbose=args.verbose
    )


if __name__ == "__main__":
    main()



# Copyright 2016, 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from config_tempest.constants import LOG
from tempest.lib import exceptions


class Users(object):
    def __init__(self, projects_client, roles_client, users_client, conf):
        """Init.

        :type projects_client: ProjectsClient object
        :type roles_client: RolesClient object from tempest lib
        :type users_client: UsersClient object from tempest lib
        :type conf: TempestConf object
        """
        self.projects_client = projects_client
        self.roles_client = roles_client
        self.users_client = users_client
        self._conf = conf

    def create_tempest_users(self):
        """Create users necessary for Tempest if they don't exist already.

        """
        sec = 'identity'
        self.create_user_with_project(self._conf.get(sec, 'username'),
                                      self._conf.get(sec, 'password'),
                                      self._conf.get(sec, 'project_name'))

        self.create_user_with_project(self._conf.get(sec, 'alt_username'),
                                      self._conf.get(sec, 'alt_password'),
                                      self._conf.get(sec, 'alt_project_name'))

        username = self._conf.get_defaulted('auth', 'admin_username')

        self.give_role_to_user(username, role_name='admin')

    def give_role_to_user(self, username, role_name,
                          role_required=True):
        """Give the user a role in the project.

        :type username: string
        :type role_name: string
        :type role_required: boolean
        """
        project_name = self._conf.get('identity', 'project_name')
        proj_id = self.projects_client.get_project_by_name(project_name)['id']
        users = self.users_client.list_users()
        user_ids = [u['id'] for u in users['users'] if u['name'] == username]
        if not user_ids:
            raise Exception("user %s not found" % username)
        user_id = user_ids[0]

        roles = self.roles_client.list_roles()
        self.check_user_roles(roles)
        role_ids = [r['id'] for r in roles['roles'] if r['name'] == role_name]
        if not role_ids:
            if role_required:
                raise Exception("required role %s not found" % role_name)
            LOG.debug("%s role not required", role_name)
            return
        role_id = role_ids[0]
        try:
            self.roles_client.create_user_role_on_project(proj_id, user_id,
                                                          role_id)
            LOG.debug("User '%s' was given the '%s' role in project '%s'",
                      username, role_name, project_name)
        except exceptions.Conflict:
            LOG.debug("(no change) User '%s' already has the '%s' role in"
                      " project '%s'", username, role_name, project_name)

    def check_user_roles(self, roles):
        """Check if roles provided by user (or the default one) exist.

        :param roles: value returned by roles_client.list_roles
        :type roles: dict
        :return: List of the existing roles given by user (or by defaults)
        :rtype: list
        """
        roles_names = [r['name'] for r in roles['roles']]
        user_roles = self._conf.get('auth', 'tempest_roles').split(',')
        available_roles = []
        for r in user_roles:
            if r in roles_names:
                available_roles.append(r)
            else:
                LOG.debug("Provided %s role is not present in the system.", r)

        if len(available_roles) == 0:
            # try 'member' or 'Member', they might present in a system
            if 'member' in roles_names:
                self._conf.set('auth', 'tempest_roles', 'member')
            elif 'Member' in roles_names:
                self._conf.set('auth', 'tempest_roles', 'Member')
            else:
                LOG.debug("Setting auth.tempest_roles to an empty list "
                          "because none of the provided roles exists.")
                self._conf.set('auth', 'tempest_roles', "")
        return available_roles

    def create_user_with_project(self, username, password, project_name):
        """Create a user and a project if it doesn't exist.

        :type username: string
        :type password: string
        :type project_name: string
        """
        LOG.info("Creating user '%s' with project '%s' and password '%s'",
                 username, project_name, password)
        project_description = "Project for Tempest %s user" % username
        email = "%s@test.com" % username
        # create a project
        try:
            self.projects_client.create_project(
                name=project_name, description=project_description)
        except exceptions.Conflict:
            LOG.info("(no change) Project '%s' already exists", project_name)

        proj_id = self.projects_client.get_project_by_name(project_name)['id']

        params = {'name': username, 'password': password,
                  'tenantId': proj_id, 'email': email}
        # create a user
        try:
            self.users_client.create_user(**params)
        except exceptions.Conflict:
            LOG.info("User '%s' already exists.", username)



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from config_tempest.services.base import VersionedService
import json


class LoadBalancerService(VersionedService):
    def set_versions(self):
        super(LoadBalancerService, self).set_versions(top_level=False)

    def set_default_tempest_options(self, conf):
        conf.set('load_balancer', 'enable_security_groups', 'True')
        conf.set('load_balancer', 'admin_role', 'admin')
        conf.set('load_balancer', 'RBAC_test_type', 'owner_or_admin')

    @staticmethod
    def get_service_type():
        return ['load-balancer']

    @staticmethod
    def get_codename():
        return 'octavia'

    def list_drivers(self):
        """List lbaas drivers"""
        body = self.do_get(self.service_url + '/v2/lbaas/providers')
        body = json.loads(body)
        names = [
            '{p[name]}:{p[description]}'.format(p=i) for i in body['providers']
        ]
        return names

    def post_configuration(self, conf, is_service):
        if not conf.has_option('auth', 'tempest_roles') \
                or conf.get('auth', 'tempest_roles') in ['', None]:
            conf.set('load_balancer', 'member_role', 'member')
        else:
            conf.set('load_balancer', 'member_role',
                     conf.get('auth', 'tempest_roles').split(',')[0])
        conf.set('load_balancer', 'region', conf.get('identity', 'region'))
        conf.set('load_balancer',
                 'enabled_provider_drivers',
                 ','.join(self.list_drivers()))



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from config_tempest.services.network import NetworkService
from config_tempest.tempest_conf import TempestConf
from config_tempest.tests.base import BaseServiceTest


class TestNetworkService(BaseServiceTest):

    FAKE_NETWORK_LIST = {
        'networks': [{
            'provider:physical_network': None,
            'id': '1ea533d7-4c65-4f25',
            'router:external': True,
            'availability_zone_hints': [],
            'availability_zones': [],
            'ipv4_address_scope': None,
            'status': 'ACTIVE',
            'subnets': ['fake_subnet'],
            'label': 'my_fake_label',
            'name': 'tempest-network',
            'admin_state_up': True,
        }]
    }

    def setUp(self):
        super(TestNetworkService, self).setUp()
        self.conf = TempestConf()
        self.Service = NetworkService("ServiceName",
                                      "ServiceType",
                                      self.FAKE_URL,
                                      self.FAKE_TOKEN,
                                      disable_ssl_validation=False)
        self.Service.client = self.FakeServiceClient()

    def test_set_get_extensions(self):
        exp_resp = ['NMN', 'OS-DCF']
        self._set_get_extensions(self.Service, exp_resp, self.FAKE_EXTENSIONS)

    def test_tempest_network_id_not_found(self):
        return_mock = mock.Mock(return_value={"networks": []})
        self.Service.client.list_networks = return_mock
        self.Service._public_network_id = "doesn't_exist"
        self.assertRaises(ValueError,
                          self.Service._supplied_network)

    def test_create_network_id_supplied_by_user(self):
        return_mock = mock.Mock(return_value=self.FAKE_NETWORK_LIST)
        self.Service.client.list_networks = return_mock
        self.Service._public_network_id = '1ea533d7-4c65-4f25'
        self.Service._supplied_network()
        self.assertEqual(self.Service._public_network_name, 'tempest-network')

    def test_create_network_auto_discover(self):
        return_mock = mock.Mock(return_value=self.FAKE_NETWORK_LIST)
        self.Service.client.list_networks = return_mock
        self.Service._discover_network()
        self.assertEqual(self.Service._public_network_id, '1ea533d7-4c65-4f25')
        self.assertEqual(self.Service._public_network_name, 'tempest-network')

    @mock.patch('config_tempest.services.network.LOG')
    def test_create_network_auto_discover_not_found(self, mock_logging):
        # delete subnets => network will not be found
        self.FAKE_NETWORK_LIST['networks'][0]['subnets'] = []
        return_mock = mock.Mock(return_value=self.FAKE_NETWORK_LIST)
        self.Service.client.list_networks = return_mock
        self.Service._discover_network()
        # check if LOG.error was called
        self.assertTrue(mock_logging.error.called)



# -*- coding: utf-8 -*-

# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from fixtures import MonkeyPatch

from config_tempest.clients import ClientManager
from config_tempest.clients import ProjectsClient
from config_tempest.tests.base import BaseConfigTempestTest
from tempest.lib import exceptions


class TestProjectsClient(BaseConfigTempestTest):
    LIST_PROJECTS = [{'name': 'my_name'}]

    def setUp(self):
        super(TestProjectsClient, self).setUp()
        self.conf = self._get_conf("v2.0", "v3")
        self.creds = self._get_creds(self.conf)
        # self.projects_manager = ProjectsClient(self.conf, self.creds)
        self.client_manager = ClientManager(self.conf, self.creds)

    def _get_projects_client(self, identity_version):
        return ProjectsClient(
            self.client_manager.auth_provider,
            self.conf.get_defaulted('identity', 'catalog_type'),
            self.client_manager.identity_region,
            'publicURL',
            identity_version,
            **self.creds.get_ssl_certificate_validation())

    def test_init(self):
        resp = self._get_projects_client('v2')
        self.assertEqual(type(resp.client).__name__, 'TenantsClient')
        # use v3 identity version and check if ProjectClient is instantiated
        resp = self._get_projects_client('v3')
        self.assertEqual(type(resp.client).__name__, 'ProjectsClient')

    @mock.patch('tempest.lib.services.identity.v2.tenants_client.'
                'TenantsClient.list_tenants')
    def test_get_project_by_name_v2(self, mock_list_tenant):
        client = self._get_projects_client('v2')
        mock_list_tenant.return_value = {'tenants': self.LIST_PROJECTS}
        resp = client.get_project_by_name('my_name')
        self.assertEqual(resp['name'], 'my_name')

    @mock.patch('tempest.lib.services.identity.v3.projects_client.'
                'ProjectsClient.list_projects')
    def test_get_project_by_name_v3(self, mock_list_project):
        client = self._get_projects_client('v3')
        mock_list_project.return_value = {'projects': self.LIST_PROJECTS}
        resp = client.get_project_by_name('my_name')
        self.assertEqual(resp['name'], 'my_name')

    @mock.patch('tempest.lib.services.identity.v3.projects_client.'
                'ProjectsClient.list_projects')
    def test_get_project_by_name_not_found(self, mock_list_project):
        client = self._get_projects_client('v3')
        mock_list_project.return_value = {'projects': self.LIST_PROJECTS}
        try:
            client.get_project_by_name('doesnt_exist')
        except exceptions.NotFound:
            # expected behaviour
            pass

    @mock.patch('tempest.lib.services.identity.v2.tenants_client.'
                'TenantsClient.create_tenant')
    def test_create_project_v2(self, mock_create_tenant):
        client = self._get_projects_client('v2')
        client.create_project('name', 'description')
        mock_create_tenant.assert_called_with(
            name='name', description='description')

    @mock.patch('tempest.lib.services.identity.v3.projects_client.'
                'ProjectsClient.create_project')
    def test_create_project_v3(self, mock_create_project):
        client = self._get_projects_client('v3')
        client.create_project('name', 'description')
        mock_create_project.assert_called_with(
            name='name', description='description')


class TestClientManager(BaseConfigTempestTest):

    def setUp(self):
        super(TestClientManager, self).setUp()
        self.conf = self._get_conf("v2.0", "v3")
        self.creds = self._get_creds(self.conf)
        # self.client = self._get_clients(self.conf)  # ?
        self.client_manager = ClientManager(self.conf, self.creds)

    def test_init_manager_as_admin(self):
        self.creds = self._get_creds(self.conf, True)
        mock_function = mock.Mock(return_value={"id": "my_fake_id"})
        func2mock = ('config_tempest.clients.ProjectsClient.'
                     'get_project_by_name')
        self.useFixture(MonkeyPatch(func2mock, mock_function))
        ClientManager(self.conf, self.creds)
        # check if admin project id was set
        admin_project_id = self.conf.get("auth", "admin_project_id")
        self.assertEqual(admin_project_id, "my_fake_id")

    def test_get_service_client(self):
        resp = self.client_manager.get_service_client('image')
        self.assertEqual(resp, self.client_manager.images)
        resp = self.client_manager.get_service_client('network')
        self.assertEqual(resp, self.client_manager)
        resp = self.client_manager.get_service_client('doesnt_exist')
        self.assertEqual(resp, None)

    def test_set_users_client(self):
        self.client_manager.users = None
        self.client_manager.set_users_client(
            self.client_manager.auth_provider,
            self.creds.identity_version,
            self.conf.get_defaulted('identity', 'catalog_type'),
            'publicURL',
            self.creds.get_ssl_certificate_validation())
        self.assertEqual(
            type(self.client_manager.users).__name__,
            'UsersClient')

    def test_set_roles_client(self):
        self.client_manager.roles = None
        self.client_manager.set_roles_client(
            self.client_manager.auth_provider,
            self.creds.identity_version,
            self.conf.get_defaulted('identity', 'catalog_type'),
            'publicURL',
            self.creds.get_ssl_certificate_validation())
        self.assertEqual(
            type(self.client_manager.roles).__name__,
            'RolesClient')



# Copyright 2013 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import re
import urllib3

from six.moves import urllib

from config_tempest.constants import LOG

from tempest.lib import exceptions


MULTIPLE_SLASH = re.compile(r'/+')


class ServiceError(Exception):
    pass


class Service(object):
    def __init__(self, name, s_type, service_url, token,
                 disable_ssl_validation, client=None, **kwargs):
        self.name = name
        self.s_type = s_type
        self.service_url = service_url
        self.headers = {'Accept': 'application/json', 'X-Auth-Token': token}
        self.disable_ssl_validation = disable_ssl_validation
        self.ca_certs = kwargs.get('ca_certs', None)
        self.client = client

        self.extensions = []
        self.versions = []
        self.versions_body = {'versions': []}

    def do_get(self, url, top_level=False, top_level_path=""):
        parts = list(urllib.parse.urlparse(url))
        # 2 is the path offset
        if top_level:
            parts[2] = '/' + top_level_path

        parts[2] = MULTIPLE_SLASH.sub('/', parts[2])
        url = urllib.parse.urlunparse(parts)

        try:
            if self.disable_ssl_validation:
                urllib3.disable_warnings()
                http = urllib3.PoolManager(cert_reqs='CERT_NONE')
            else:
                http = urllib3.PoolManager()
            r = http.request('GET', url, headers=self.headers)
        except Exception as e:
            LOG.error("Request on service '%s' with url '%s' failed",
                      self.s_type, url)
            raise e

        if r.status == 403:
            raise exceptions.Forbidden("Request on service '%s' with url '%s' "
                                       "failed with code 403" % (self.s_type,
                                                                 url))

        if r.status >= 400:
            raise ServiceError("Request on service '%s' with url '%s' failed"
                               " with code %d" % (self.s_type, url, r.status))
        return r.data.decode('utf-8')

    def set_extensions(self):
        self.extensions = []

    def set_versions(self):
        self.versions = []

    def set_availability(self, conf, available):
        """Sets service's availability.

        The services's codename will be set to desired value under
        [service_available] section in tempest.conf during the services
        discovery process.
        """
        try:
            conf.set('service_available', self.get_codename(), str(available))
        except NotImplementedError:
            pass

    def get_extensions(self):
        return self.extensions

    @staticmethod
    def get_service_type():
        """Return the service type.

        This returns a list because you can have services with more types,
        like volume, volumev2, volumev3.
        """
        return []

    def get_versions(self):
        """Return the versions available for each service.

        This doesn't mean tempestconf supports all these versions. Only that
        the service has these api versions enabled.
        """
        return self.versions

    def set_default_tempest_options(self, conf):
        pass

    def get_supported_versions(self):
        """Return the versions supported by tempestconf.

        The server might have older or newer versions that could not be
        supported by tempestconf.
        """
        return []

    @staticmethod
    def get_codename():
        """Return the service_available name of the service.

        This name is used when setting service availability in
        set_availability method. If the method is not implemented, service
        availability is not set.
        """
        raise NotImplementedError

    def get_feature_name(self):
        """Return the name of service used in <service>-feature-enabled.

        Some services have the -feature-enabled option in tempest, that
        diverges from the service name. The main example is object-store
        service where the <service>-feature-enabled is object-storage.
        """
        return self.s_type

    def get_service_extension_key(self):
        """Return the extension key for a particular service"""
        return None

    def get_unversioned_service_type(self):
        """Return type of service without versions.

        Some services are versioned like volumev2 and volumev3, we try to
        discover these services checking the supported versions, so we need
        to know the unversioned service type for this.
        The default value is the type of the service.
        """
        return self.s_type

    def post_configuration(self, conf, is_service):
        """Do post congiruation steps.

        :param conf: config_tempest.tempest_conf.TempestConf
        :param is_service: config_tempest.services.services.Services.is_service
        """
        return None


class VersionedService(Service):
    def set_versions(self, top_level=True):
        body = self.do_get(self.service_url, top_level=top_level)
        self.versions_body = json.loads(body)
        self.versions = self.deserialize_versions(self.versions_body)

    def deserialize_versions(self, body):
        versions = []
        for version in body['versions']:
            if version['status'] != "DEPRECATED":
                versions.append(version)
        return list(map(lambda x: x['id'], versions))

    def filter_api_microversions(self, max_version='version'):
        min_microversion = ''
        max_microversion = ''
        for version in self.versions_body['versions']:
            if version['status'] != "DEPRECATED":
                if max_microversion == '':
                    max_microversion = version[max_version]
                else:
                    max_microversion = max(max_microversion,
                                           version[max_version])
                if 'min_version' not in version:
                    continue
                if min_microversion == '':
                    min_microversion = version['min_version']
                else:
                    min_microversion = min(min_microversion,
                                           version['min_version'])
        return {'max_microversion': max_microversion,
                'min_microversion': min_microversion}

    def no_port_cut_url(self):
        # if there is no port defined, cut the url from version to the end
        u = urllib3.util.parse_url(self.service_url)
        url = self.service_url
        if u.port is None:
            found = re.findall(r'v\d', url)
            if len(found) > 0:
                index = url.index(found[0])
                url = self.service_url[:index]
        return (url, u.port is not None)



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from operator import itemgetter

from config_tempest import constants as C


class Flavors(object):
    def __init__(self, client, allow_creation, conf, min_memory, min_disk,
                 no_rng=False):
        """Init.

        :type client: FlavorsClient object from tempest lib
        :type allow_creation: boolean
        :type conf: TempestConf object
        """
        self.client = client
        self.allow_creation = allow_creation
        self._conf = conf
        self.flavor_list = self.client.list_flavors()['flavors']
        min_memory_alt = C.DEFAULT_FLAVOR_RAM_ALT
        name = 'm1.nano'
        name_alt = 'm1.micro'
        if min_memory != C.DEFAULT_FLAVOR_RAM:
            min_memory_alt = min_memory + 1
            name = 'custom'
            name_alt = 'custom_alt'
        self._conf.set('volume', 'volume_size', str(min_disk))
        self.prefs = [
            {'key': 'flavor_ref', 'name': name, 'ram': min_memory,
             'disk': min_disk, 'no_rng': no_rng},
            {'key': 'flavor_ref_alt', 'name': name_alt,
             'ram': min_memory_alt, 'disk': min_disk, 'no_rng': no_rng}
        ]

    def create_tempest_flavors(self):
        """Find or create flavors and set them in conf.

        If 'flavor_ref' and 'flavor_ref_alt' are specified in conf, it will
        try to find them, if not found, it raises an Exception.
        Otherwise it will try finding or creating the required base flavors
        (m1.nano and m1.micro by default) and set their ids in conf.
        """
        for pref in self.prefs:
            flavor_id = None
            if self._conf.has_option('compute', pref['key']):
                flavor_id = self._conf.get('compute', pref['key'])
                flavor_id = self.find_flavor_by_id(flavor_id)
                if flavor_id is None:
                    raise Exception("%s id '%s' specified by user doesn't"
                                    " exist", pref['key'], flavor_id)
            else:
                flavor_id = self.create_flavor(pref['name'], pref['ram'],
                                               C.DEFAULT_FLAVOR_VCPUS,
                                               pref['disk'],
                                               no_rng=pref['no_rng'])
                self._conf.set('compute', pref['key'], flavor_id)

    def create_flavor(self, flavor_name, ram, vcpus, disk, no_rng=False):
        """Create flavors or try to discover two smallest ones available.

        :param flavor_name: flavor name to be created (usually m1.nano or
                            m1.micro)
        :param ram: memory of created flavor in MB
        :param vcpus: number of VCPUs for the flavor
        :param disk: size of disk for flavor in GB
        :param no_rng: boolean, if True, flavor will be created with RNG device
        """
        flavor_id = self.find_flavor_by_name(flavor_name)
        if flavor_id is not None:
            C.LOG.info("(no change) Found flavor '%s'", flavor_name)
            return flavor_id
        elif self.allow_creation:
            C.LOG.info("Creating flavor '%s'", flavor_name)
            resp = self.client.create_flavor(name=flavor_name,
                                             ram=ram, vcpus=vcpus,
                                             disk=disk, id=None)
            args = {'flavor_id': resp['flavor']['id'],
                    'hw_rng:allowed': 'True'}
            if no_rng:
                args.pop('hw_rng:allowed')
            self.client.set_flavor_extra_spec(**args)
            return resp['flavor']['id']
        else:
            if len(self.flavor_list) < 2:
                raise Exception("Creation of flavors is not allowed and not "
                                "enough available flavors found. Either use --"
                                "create argument or create flavors manually.")
            # return id of the discovered flavor
            return self.discover_smallest_flavor(flavor_name)

    def find_flavor_by_id(self, flavor_id):
        """Look for a flavor by its id.

        :type flavor_id: string
        :return: flavor id or None if not found
        :rtype: string or None
        """
        found = [f for f in self.flavor_list if f['id'] == flavor_id]
        if found:
            C.LOG.info("Found flavor '%s' by it's id '%s'",
                       found[0]['name'], flavor_id)
            # return flavor's id
            return found[0]['id']
        return None

    def find_flavor_by_name(self, flavor_name):
        """Look for a flavor by its name.

        :type flavor_name: string
        :return: flavor id or None if not found
        :rtype: string or None
        """
        found = [f for f in self.flavor_list if f['name'] == flavor_name]
        if found:
            # return flavor's id
            return found[0]['id']
        return None

    def discover_smallest_flavor(self, flavor_name=""):
        """Discover the two smallest available flavors in the system.

        If flavor_name contains "micro", the method returns the second
        smallest flavor found.
        :param flavor_name: [m1.nano, m1.micro]
        """
        C.LOG.warning("Flavor '%s' not found and creation is not allowed. "
                      "Trying to autodetect the smallest flavor available.",
                      flavor_name)
        flavors = []
        for flavor in self.flavor_list:
            f = self.client.show_flavor(flavor['id'])['flavor']
            flavors.append((f['name'], f['id'], f['ram'],
                            f['disk'], f['vcpus']))

        # order by ram, disk size and vcpus number and take first two of them
        flavors = sorted(flavors, key=itemgetter(2, 3, 4))[:2]

        f = None
        if "micro" in flavor_name:
            # take the second smaller one
            f = flavors[1]
        else:
            f = flavors[0]
        C.LOG.warning("Found '%s' flavor (id: '%s', ram: '%s', disk: '%s', "
                      "vcpus: '%s') ", f[0], f[1], f[2], f[3], f[4])
        # return flavor's id
        return f[1]



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from fixtures import MonkeyPatch

from config_tempest.services.identity import IdentityService
from config_tempest.tempest_conf import TempestConf
from config_tempest.tests.base import BaseServiceTest


class TestIdentityService(BaseServiceTest):
    def setUp(self):
        super(TestIdentityService, self).setUp()
        self.Service = IdentityService("ServiceName",
                                       "ServiceType",
                                       self.FAKE_URL + 'v2.0/',
                                       self.FAKE_TOKEN,
                                       disable_ssl_validation=False)

    def test_set_extensions(self):
        expected_resp = ['OS-DCF', 'NMN']
        self._set_get_extensions(self.Service, expected_resp,
                                 self.FAKE_IDENTITY_EXTENSIONS)
        # try with URL containing v3 version
        self.Service.service_url = self.FAKE_URL + 'v3'
        expected_resp = []
        self._set_get_extensions(self.Service, expected_resp,
                                 self.FAKE_IDENTITY_EXTENSIONS)

    def test_get_extensions(self):
        exp_resp = ['OS-INHERIT', 'OS-OAUTH1',
                    'OS-SIMPLE-CERT', 'OS-EP-FILTER']
        self.Service.extensions = exp_resp[:2]
        self.Service.extensions_v3 = exp_resp[2:]
        self.assertItemsEqual(self.Service.get_extensions(), exp_resp)

    def test_set_identity_v3_extensions(self):
        expected_resp = ['OS-INHERIT', 'OS-OAUTH1',
                         'OS-SIMPLE-CERT', 'OS-EP-FILTER']
        fake_resp = self.FakeRequestResponse(bytes_content=True)
        mocked_requests = mock.Mock()
        mocked_requests.return_value = fake_resp
        self.useFixture(MonkeyPatch('requests.get', mocked_requests))
        self.Service.service_url = self.FAKE_URL + "v3"
        fake_resp.raise_for_status = mock.Mock()
        self.Service.set_identity_v3_extensions()
        self.assertItemsEqual(self.Service.extensions_v3, expected_resp)
        self.assertItemsEqual(self.Service.get_extensions(), expected_resp)

    def test_set_get_versions(self):
        exp_resp = ['v3.8']
        self._set_get_versions(self.Service, exp_resp,
                               self.FAKE_IDENTITY_VERSIONS)

    def test_deserialize_versions(self):
        expected_resp = ['v3.8']
        self._test_deserialize_versions(self.Service,
                                        expected_resp,
                                        self.FAKE_IDENTITY_VERSIONS)
        self._test_deserialize_versions(self.Service,
                                        expected_resp,
                                        self.FAKE_IDENTITY_VERSION)
        expected_resp = ['v2.1', 'v3.8']
        # add not deprecated v2 version to FAKE_IDENTITY_VERSIONS
        v2 = [{'status': 'stable', 'id': 'v2.1'}]
        versions = self.FAKE_IDENTITY_VERSIONS['versions']['values'] + v2
        fake_versions = {'versions': {'values': versions}}
        self._test_deserialize_versions(self.Service,
                                        expected_resp,
                                        fake_versions)

    @mock.patch('config_tempest.services.identity'
                '.IdentityService.get_versions')
    def test_set_default_tempest_options(self, mock_get_versions):
        conf = TempestConf()
        mock_get_versions.return_value = ['v3.8', 'v2.0']
        self.Service.set_default_tempest_options(conf)
        self.assertEqual(
            conf.get('identity-feature-enabled',
                     'forbid_global_implied_dsr'), 'True')
        self.assertEqual(
            conf.get('identity', 'auth_version'), 'v3')
